#include "dlgproperties.h"

#include <qvariant.h>
#include <qcheckbox.h>
#include <qcombobox.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qpushbutton.h>
#include <qtabwidget.h>
#include <qtoolbutton.h>
#include <qwidget.h>
#include <qlayout.h>
#include <qtooltip.h>
#include <qwhatsthis.h>
#include <qimage.h>
#include <qpixmap.h>
#include <qapp.h>
#include <qmessagebox.h>

static const char* const image0_data[] = {
"14 14 4 1",
". c None",
"b c #000400",
"# c #838100",
"a c #bdc2bd",
"..............",
".#.aaaaaaaa.a.",
".#.aaaaaaaa...",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".#.aaaaaaaa.#.",
".##bbbbbbbb##.",
".############.",
".##.........#.",
".##......aa.#.",
".##......aa.#.",
".##......aa.#.",
"a............."};

/* 
 *  Constructs a dlgProperties which is a child of 'parent', with the 
 *  name 'name' and widget flags set to 'f'.
 *
 *  The dialog will by default be modeless, unless you set 'modal' to
 *  TRUE to construct a modal dialog.
 */
dlgProperties::dlgProperties(QFileInfo inf, QWidget* parent,  const char* name, bool modal, WFlags fl )
    : QDialog( parent, name, modal, fl )
{
    someThingChanged = false;
    QPixmap image0( ( const char** ) image0_data );
    if ( !name )
    	setName( "dlgProperties" );
    isDir = inf.isDir();
    resize( 604, 382 );
    setCaption( trUtf8( "Properties for selected file   " ) + inf.absFilePath() );
    fileName = inf.absFilePath();
    info = inf;

    setIcon( image0 );
    dlgPropertiesLayout = new QGridLayout( this, 1, 1, 11, 6, "dlgPropertiesLayout"); 

    PushButton3 = new kcSimpleButton( this, "PushButton3" );
    PushButton3->setMaximumSize( QSize( 32767, 30 ) );
    PushButton3->setText( trUtf8( "Ok" ) );

    dlgPropertiesLayout->addWidget( PushButton3, 1, 1 );
    QSpacerItem* spacer = new QSpacerItem( 240, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
    dlgPropertiesLayout->addItem( spacer, 1, 0 );

    TabWidget2 = new QTabWidget( this, "TabWidget2" );

    tab = new QWidget( TabWidget2, "tab" );
    tabLayout = new QGridLayout( tab, 1, 1, 11, 6, "tabLayout"); 

    LineEdit = new QLineEdit( tab, "LineEdit" );
    LineEdit->setMaximumSize( QSize( 32767, 25 ) );

    tabLayout->addMultiCellWidget( LineEdit, 0, 0, 1, 2 );

    ToolButton = new QToolButton( tab, "ToolButton" );
    ToolButton->setMinimumSize( QSize( 32, 32 ) );
    ToolButton->setMaximumSize( QSize( 32, 32 ) );

    if(inf.isFile())
      ToolButton->setPixmap(kcFileIconProvider::getPixmap(inf.absFilePath()));
    else
      ToolButton->setPixmap(kcFileIconProvider::getPixmap("folder"));
    ToolButton->setUsesBigPixmap( TRUE );

    tabLayout->addWidget( ToolButton, 0, 0 );

    LabelTyp_2 = new QLabel( tab, "LabelTyp_2" );
    LabelTyp_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelTyp_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelTyp_2, 2, 2, 1, 2 );

    LabelSize_2 = new QLabel( tab, "LabelSize_2" );
    LabelSize_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelSize_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelSize_2, 4, 4, 1, 2 );

    LabelAdr_2 = new QLabel( tab, "LabelAdr_2" );
    LabelAdr_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelAdr_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelAdr_2, 3, 3, 1, 2 );

    LabelTyp_1 = new QLabel( tab, "LabelTyp_1" );
    LabelTyp_1->setMaximumSize( QSize( 220, 20 ) );
    LabelTyp_1->setText( trUtf8( "Type:" ) );

    tabLayout->addWidget( LabelTyp_1, 2, 0 );

    LabelAdr_1 = new QLabel( tab, "LabelAdr_1" );
    LabelAdr_1->setMaximumSize( QSize( 220, 20 ) );
    LabelAdr_1->setText( trUtf8( "Location:" ) );

    tabLayout->addWidget( LabelAdr_1, 3, 0 );

    LabelSize_1 = new QLabel( tab, "LabelSize_1" );
    LabelSize_1->setMaximumSize( QSize( 220, 20 ) );
    LabelSize_1->setText( trUtf8( "Size:" ) );

    tabLayout->addWidget( LabelSize_1, 4, 0 );
//    QSpacerItem* spacer_2 = new QSpacerItem( 220, 20, QSizePolicy::Fixed, QSizePolicy::Minimum );
//    tabLayout->addItem( spacer_2, 5, 0 );

//    PushButtonNew = new kcSimpleButton( tab, "PushButtonNew" );
//    PushButtonNew->setMaximumSize( QSize( 32767, 22 ) );
//    PushButtonNew->setText( trUtf8( "Calc size" ) );
//    PushButtonNew->setAccel( 0 );
//
//    tabLayout->addWidget( PushButtonNew, 5, 1 );
//
//    PushButtonHalt = new kcSimpleButton( tab, "PushButtonHalt" );
//    PushButtonHalt->setEnabled( FALSE );
//    PushButtonHalt->setMaximumSize( QSize( 32767, 22 ) );
//    PushButtonHalt->setText( trUtf8( "Stop" ) );
//
//    tabLayout->addWidget( PushButtonHalt, 5, 2 );

    LabelFree_1 = new QLabel( tab, "LabelFree_1" );
    LabelFree_1->setMaximumSize( QSize( 220, 20 ) );
    LabelFree_1->setText( trUtf8( "Free space at .." ) );

    tabLayout->addWidget( LabelFree_1, 6, 0 );

    LabelChanged_1 = new QLabel( tab, "LabelChanged_1" );
    LabelChanged_1->setMaximumSize( QSize( 220, 20 ) );
    LabelChanged_1->setText( trUtf8( "Last change:" ) );

    tabLayout->addWidget( LabelChanged_1, 8, 0 );

    LabelLast_1 = new QLabel( tab, "LabelLast_1" );
    LabelLast_1->setMaximumSize( QSize( 220, 20 ) );
    LabelLast_1->setText( trUtf8( "Last access:" ) );

    tabLayout->addWidget( LabelLast_1, 9, 0 );

    LabelFree_2 = new QLabel( tab, "LabelFree_2" );
    LabelFree_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelFree_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelFree_2, 6, 6, 1, 2 );

    LabelChanged_2 = new QLabel( tab, "LabelChanged_2" );
    LabelChanged_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelChanged_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelChanged_2, 8, 8, 1, 2 );

    LabelLast_2 = new QLabel( tab, "LabelLast_2" );
    LabelLast_2->setMaximumSize( QSize( 32767, 20 ) );
    LabelLast_2->setText( trUtf8( "TextLabel3" ) );

    tabLayout->addMultiCellWidget( LabelLast_2, 9, 9, 1, 2 );

    Line1 = new QFrame( tab, "Line1" );
    Line1->setProperty( "frameShape", (int)QFrame::HLine );
    Line1->setFrameShadow( QFrame::Sunken );
    Line1->setFrameShape( QFrame::HLine );

    tabLayout->addMultiCellWidget( Line1, 1, 1, 0, 2 );

    Line2 = new QFrame( tab, "Line2" );
    Line2->setProperty( "frameShape", (int)QFrame::HLine );
    Line2->setFrameShadow( QFrame::Sunken );
    Line2->setFrameShape( QFrame::HLine );

    tabLayout->addMultiCellWidget( Line2, 7, 7, 0, 2 );
    TabWidget2->insertTab( tab, trUtf8( "General" ) );

    tab_2 = new QWidget( TabWidget2, "tab_2" );
    tabLayout_2 = new QGridLayout( tab_2, 1, 1, 11, 6, "tabLayout_2"); 

    CkeckKeepSubdirs = new QCheckBox( tab_2, "CkeckKeepSubdirs" );
    CkeckKeepSubdirs->setMaximumSize( QSize( 32767, 22 ) );
    CkeckKeepSubdirs->setText( trUtf8( "Keep changes to subdirectories and the content" ) );
    CkeckKeepSubdirs->setEnabled(false);

    tabLayout_2->addWidget( CkeckKeepSubdirs, 2, 0 );

    GroupBox_2 = new QGroupBox( tab_2, "GroupBox_2" );
    GroupBox_2->setTitle( trUtf8( "Owner" ) );

    LabelUser_2 = new QLabel( GroupBox_2, "LabelUser_2" );
    LabelUser_2->setGeometry( QRect( 16, 35, 85, 22 ) ); 
    LabelUser_2->setMaximumSize( QSize( 32767, 22 ) );
    LabelUser_2->setText( trUtf8( "User" ) );

    LabelGroup_2 = new QLabel( GroupBox_2, "LabelGroup_2" );
    LabelGroup_2->setGeometry( QRect( 16, 65, 85, 22 ) ); 
    LabelGroup_2->setMaximumSize( QSize( 32767, 22 ) );
    LabelGroup_2->setText( trUtf8( "Group" ) );

    ComboChgrp = new QComboBox( FALSE, GroupBox_2, "ComboChgrp" );
//    ComboChgrp->insertItem( trUtf8( "users" ) );
    ComboChgrp->setGeometry( QRect( 106, 65, 110, 21 ) ); 

    LabelUser_3 = new QLabel( GroupBox_2, "LabelUser_3" );
    LabelUser_3->setGeometry( QRect( 106, 35, 85, 22 ) ); 
    LabelUser_3->setMaximumSize( QSize( 32767, 22 ) );
    LabelUser_3->setText( trUtf8( "cfricke" ) );

    tabLayout_2->addWidget( GroupBox_2, 1, 0 );

    GroupBox_1 = new QGroupBox( tab_2, "GroupBox_1" );
    GroupBox_1->setTitle( trUtf8( "Rights" ) );

    CheckBox_Owner_Execute = new QCheckBox( GroupBox_1, "CheckBox_Owner_Execute" );
    CheckBox_Owner_Execute->setGeometry( QRect( 260, 50, 16, 21 ) );
    CheckBox_Owner_Execute->setText( trUtf8( "CheckBox1" ) );

    CheckBox_Group_Execute = new QCheckBox( GroupBox_1, "CheckBox_Other_Execute" );
    CheckBox_Group_Execute->setGeometry( QRect( 260, 80, 16, 21 ) );
    CheckBox_Group_Execute->setText( trUtf8( "CheckBox2" ) );

    CheckBox_Other_Execute = new QCheckBox( GroupBox_1, "CheckBox_Group_Execute" );
    CheckBox_Other_Execute->setGeometry( QRect( 260, 110, 16, 21 ) );
    CheckBox_Other_Execute->setText( trUtf8( "CheckBox3" ) );

    CheckBox_Owner_Write = new QCheckBox( GroupBox_1, "CheckBox_Owner_Write" );
    CheckBox_Owner_Write->setGeometry( QRect( 180, 50, 16, 21 ) );
    CheckBox_Owner_Write->setText( trUtf8( "CheckBox4" ) );

    CheckBox_Group_Write = new QCheckBox( GroupBox_1, "CheckBox_Group_Write" );
    CheckBox_Group_Write->setGeometry( QRect( 180, 80, 16, 21 ) );
    CheckBox_Group_Write->setText( trUtf8( "CheckBox5" ) );

    CheckBox_Other_Write = new QCheckBox( GroupBox_1, "CheckBox_Other_Write" );
    CheckBox_Other_Write->setGeometry( QRect( 180, 110, 16, 21 ) );
    CheckBox_Other_Write->setText( trUtf8( "CheckBox6" ) );

    CheckBox_Owner_Read = new QCheckBox( GroupBox_1, "CheckBox_Owner_Read" );
    CheckBox_Owner_Read->setGeometry( QRect( 100, 50, 16, 21 ) );
    CheckBox_Owner_Read->setText( trUtf8( "CheckBox7" ) );

    CheckBox_Group_Read = new QCheckBox( GroupBox_1, "CheckBox_Group_Read" );
    CheckBox_Group_Read->setGeometry( QRect( 100, 80, 16, 21 ) );
    CheckBox_Group_Read->setText( trUtf8( "CheckBox8" ) );

    CheckBox_Other_Read = new QCheckBox( GroupBox_1, "CheckBox_Other_Read" );
    CheckBox_Other_Read->setGeometry( QRect( 100, 110, 16, 21 ) );
    CheckBox_Other_Read->setText( trUtf8( "CheckBox9" ) );

    LabelRead = new QLabel( GroupBox_1, "LabelRead" );
    LabelRead->setGeometry( QRect( 96, 22, 85, 22 ) ); 
    LabelRead->setMaximumSize( QSize( 32767, 22 ) );
    LabelRead->setText( trUtf8( "Read" ) );

    LabelWrite = new QLabel( GroupBox_1, "LabelWrite" );
    LabelWrite->setGeometry( QRect( 176, 22, 70, 22 ) ); 
    LabelWrite->setMaximumSize( QSize( 32767, 22 ) );
    LabelWrite->setText( trUtf8( "Write" ) );

    LabelExec = new QLabel( GroupBox_1, "LabelExec" );
    LabelExec->setGeometry( QRect( 256, 22, 85, 22 ) ); 
    LabelExec->setMaximumSize( QSize( 32767, 22 ) );
    LabelExec->setText( trUtf8( "Exec" ) );

    LabelUser_1 = new QLabel( GroupBox_1, "LabelUser_1" );
    LabelUser_1->setGeometry( QRect( 10, 50, 85, 22 ) );
    LabelUser_1->setMaximumSize( QSize( 32767, 22 ) );
    LabelUser_1->setText( trUtf8( "User" ) );

    LabelGroup = new QLabel( GroupBox_1, "LabelGroup" );
    LabelGroup->setGeometry( QRect( 10, 80, 85, 22 ) ); 
    LabelGroup->setMaximumSize( QSize( 32767, 22 ) );
    LabelGroup->setText( trUtf8( "Group" ) );

    LabelOther = new QLabel( GroupBox_1, "LabelOther" );
    LabelOther->setGeometry( QRect( 10, 110, 85, 22 ) ); 
    LabelOther->setMaximumSize( QSize( 32767, 22 ) );
    LabelOther->setText( trUtf8( "Other" ) );


    LabelSpecial = new QLabel( GroupBox_1, "LabelSpecial" );
    LabelSpecial->setGeometry( QRect( 330, 20, 85, 22 ) ); 
    LabelSpecial->setMaximumSize( QSize( 32767, 22 ) );
    LabelSpecial->setText( trUtf8( "Special" ) );

    CheckSetUID = new QCheckBox( GroupBox_1, "CheckSetUID" );
    CheckSetUID->setGeometry( QRect( 330, 50, 120, 21 ) ); 
    CheckSetUID->setText( trUtf8( "Set user ID" ) );

    CheckSetGrpID = new QCheckBox( GroupBox_1, "CheckSetGrpID" );
    CheckSetGrpID->setGeometry( QRect( 330, 80, 120, 21 ) ); 
    CheckSetGrpID->setText( trUtf8( "Set group ID" ) );

    CheckSetSticky = new QCheckBox( GroupBox_1, "CheckSetSticky" );
    CheckSetSticky->setGeometry( QRect( 330, 110, 120, 21 ) ); 
    CheckSetSticky->setText( trUtf8( "Set sticky bit" ) );

    tabLayout_2->addWidget( GroupBox_1, 0, 0 );
    TabWidget2->insertTab( tab_2, trUtf8( "Rights" ) );

    dlgPropertiesLayout->addMultiCellWidget( TabWidget2, 0, 0, 0, 2 );

    PushButton4 = new kcSimpleButton( this, "PushButton4" );
    PushButton4->setMaximumSize( QSize( 32767, 30 ) );
    PushButton4->setText( trUtf8( "Cancel" ) );

    dlgPropertiesLayout->addWidget( PushButton4, 1, 2 );

    // signals and slots connections
    connect( PushButton3, SIGNAL( clicked() ), this, SLOT( okClicked() ) );
    connect( PushButton4, SIGNAL( clicked() ), this, SLOT( reject() ) );
//    connect( PushButtonNew, SIGNAL( clicked() ), this, SLOT( slot_ReCalculate() ) );
//    connect( PushButtonHalt, SIGNAL( clicked() ), this, SLOT( slot_StopCalc() ) );
    connect( LineEdit, SIGNAL( returnPressed() ), this, SLOT( slot_RenameFile() ) );

    fillItems(inf);
    LabelFree_1->setText("Free space at  "+ getMountPoint(inf.dirPath(true)));
    dirSize(inf.absFilePath());

   struct  statfs g;
   statfs((const char*)getMountPoint(inf.dirPath(true)),&g);
   int free = g.f_bfree;
   int blocksize = g.f_bsize/1024;
   unsigned int real = (free * blocksize);
   QString size;
   size.sprintf("%u",real);
   LabelFree_2->setText(size+ " Megabyte");

   stateOfCheckBoxs = false;
   connect(CheckBox_Owner_Read,    SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Group_Read,    SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Other_Read,    SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Owner_Write,   SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Group_Write,   SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Other_Write,   SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Owner_Execute, SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Group_Execute, SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckBox_Other_Execute, SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckSetUID,            SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckSetGrpID,          SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));
   connect(CheckSetSticky,         SIGNAL(stateChanged(int)), this, SLOT(checkBoxStateChanged(int)));

   connect(ComboChgrp,             SIGNAL(activated(int)),this , SLOT(changeGroup(int)))  ;
}

/*  
 *  Destroys the object and frees any allocated resources
 */
dlgProperties::~dlgProperties()
{
    // no need to delete child widgets, Qt does it all for us
}

void dlgProperties::fillItems(QFileInfo inf)
{
  LineEdit->setText(inf.fileName());
  if(inf.isDir()) LabelTyp_2->setText("Directory");
    else LabelTyp_2->setText("File");
  LabelAdr_2->setText(inf.dirPath(true));
  if(inf.isDir())
    LabelSize_2->setText("------") ;
  else
  {
    QString s;
    unsigned int bytes = inf.size();
    if(bytes >= 1073741824)
//        s.sprintf("%d GB ", bytes/1024/1024/1024);
//      else if(bytes >= 1048576)
      s.sprintf("%d MB ", bytes/1024/1024);
    else if(bytes >= 1024)
      s.sprintf("%d KB ", bytes/1024);
    else
      s.sprintf("%d Bytes ", bytes);
    LabelSize_2->setText(s);
  }
  LabelChanged_2->setText(inf.lastModified().toString());
  LabelLast_2->setText(inf.lastRead().toString());

  if ( inf.permission( QFileInfo::ReadUser) )
      CheckBox_Owner_Read->setChecked(true);
  if ( inf.permission( QFileInfo::WriteUser) )
      CheckBox_Owner_Write->setChecked(true);
  if ( inf.permission( QFileInfo::ExeUser) )
      CheckBox_Owner_Execute->setChecked(true);

  if ( inf.permission( QFileInfo::ReadGroup) )
      CheckBox_Group_Read->setChecked(true);
  if ( inf.permission( QFileInfo::WriteGroup) )
      CheckBox_Group_Write->setChecked(true);
  if ( inf.permission( QFileInfo::ExeGroup) )
      CheckBox_Group_Execute->setChecked(true);

  if ( inf.permission( QFileInfo::ReadOther) )
      CheckBox_Other_Read->setChecked(true);
  if ( inf.permission( QFileInfo::WriteOther) )
      CheckBox_Other_Write->setChecked(true);
  if ( inf.permission( QFileInfo::ExeOther) )
      CheckBox_Other_Execute->setChecked(true);

  struct stat buf;
  stat((const char *)fileName,&buf);
  int a = buf.st_mode ;
  int s = a&S_ISVTX;
  int u = a&S_ISUID;
  int g = a&S_ISGID;

  if(u)    CheckSetUID->setChecked(true);
  if(g)    CheckSetGrpID->setChecked(true);
  if(s)    CheckSetSticky->setChecked(true);

  LabelUser_3->setText(inf.owner());
  getGroupList();
  ComboChgrp->setCurrentText(inf.group());
  newGroup = inf.group();

}

QString dlgProperties::getNameForID(uint ID)
{
    struct passwd *p_entry;//, *p_entry2;
    p_entry = getpwuid(ID);
    return(QString(p_entry->pw_name));
}

uint dlgProperties::getIDForName(QString name)
{
    struct passwd *p_entry;
    p_entry = getpwnam(name);
    return p_entry->pw_uid;
}

QString dlgProperties::getGroupName(uint ID)
{
  struct group *g_entry;
//  int i;

  g_entry = getgrgid(ID) ;
  return QString(g_entry->gr_name);
}

uint dlgProperties::getGroupId(QString name)
{
  struct group *g_entry;
  g_entry = getgrnam(name) ;
  return g_entry->gr_gid;
}

void dlgProperties::getGroupList()
{  // gleich die Box fllen ist am Besten !!
  struct group *g_entry, *g_entry1;
  int id;

  while( (g_entry=getgrent()) != 0 )
    {
      id = g_entry->gr_gid;
      g_entry1 = getgrgid(id);
      ComboChgrp->insertItem(g_entry1->gr_name);
    }
  setgrent();  // hier war das gleiche wie bei FSTAB aber so wird wieder zuruegesetzt !!
}

unsigned long int dlgProperties::dirSize(QString dirName)
{
   proc = new QProcess( this );

   proc->addArgument( "du" );
   proc->addArgument( "-s" );
   proc->addArgument( "-h" );
   proc->addArgument( "-k" );
   proc->addArgument(dirName);

   connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()) );
   proc->setCommunication(QProcess::Stdin | QProcess::Stdout | QProcess::Stderr);
   proc->start();
}
void dlgProperties::readFromStdout()
{
  QString s = proc->readLineStdout();
  s = s.left(s.find("\t")) + tr(" kb");
  LabelSize_2->setText(s);
}
void dlgProperties::duReady()
{
}

QString dlgProperties::getMountPoint(QString dir)
{
  if(dir  == "/") return QString("/");
  QString s;
  int count = dir.contains("/",true);

  for(int i = 0; i < count;i++)
  {
    struct mntent *mnt_entry;
    FILE *fp;
    if((fp = setmntent(MOUNTED, "r")))
    {
        while((mnt_entry = getmntent(fp)))
        {
            s.sprintf("%s",mnt_entry->mnt_dir);
            if (s == dir) return dir;
        }
        endmntent(fp);
    }
    dir.truncate(dir.findRev("/"));
    getMountPoint(dir);
  };

  return QString("/");
}

void dlgProperties::checkBoxStateChanged(int)
{
  stateOfCheckBoxs = true;
}

void dlgProperties::changeGroup(int index)
{
  newGroup = ComboChgrp->text(index);
  int ret = chown(fileName,getIDForName("cfricke"),getGroupId(ComboChgrp->text(index)));
  if(ret)
  {
	  if(info.owner() == "root")  QMessageBox::warning( this, "Chown failed", "This operation failed" )  ;
	  else QMessageBox::warning( this, "Chown failed", "Only superuser can do that" );
  }
  someThingChanged = true;
}

bool dlgProperties::setNewMode()
{
  int rights = 0;
  if(CheckBox_Owner_Read->isChecked())
    rights +=  0400;
  if(CheckBox_Owner_Write->isChecked())
    rights +=  0200;
  if(CheckBox_Owner_Execute->isChecked())
    rights +=  0100;

  if(CheckBox_Group_Read->isChecked())
    rights +=  0040;
  if(CheckBox_Group_Write->isChecked())
    rights +=  0020;
  if(CheckBox_Group_Execute->isChecked())
    rights +=  0010;

  if(CheckBox_Other_Read->isChecked())
    rights +=  0004;
  if(CheckBox_Other_Write->isChecked())
    rights +=  0002;
  if(CheckBox_Other_Execute->isChecked())
    rights +=  0001;

  if(CheckSetUID->isChecked())
    rights +=  04000;
  if(CheckSetGrpID->isChecked())
    rights +=  02000;
  if(CheckSetSticky->isChecked())
    rights +=  01000;

  if(isDir)
    rights += 040000;
  else
    rights += 0100000;

  if(chmod(fileName,rights))
    return true;
  else
    return false;
}

void dlgProperties::okClicked()
{
  if(stateOfCheckBoxs)
  {
    setNewMode();
    someThingChanged = true;
  }
  accept();
}
