/***************************************************************************
                          hexviewedit.h  -  description
                             -------------------
    begin                : Mon Jan 7 2002
    copyright            : (C) 2002 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#ifndef HEXVIEWEDIT_H
#define HEXVIEWEDIT_H

#include <stdio.h>
#include <stdlib.h>
#include <qwidget.h>
#include <qfile.h>
#include <qtimer.h>
#include <qstyle.h>
#include <qpainter.h>
#include <qpixmap.h>
#include <qapp.h>
#include <qfile.h>

#include <sys/stat.h>
#include <unistd.h>
#include <qscrollbar.h>

/**
  *@author Christian Fricke & Ren Mrten
  */

struct ub{
            uint offset;
            uchar byte;
            struct ub *nextNode;
            struct ub *prevNode;
            struct ub *lastNode; // for reverse (undo is allways reverse ;) )
          };

class hexViewEdit : public QWidget  {
   Q_OBJECT
public: 
	hexViewEdit(QString filename,bool editflag=false, QWidget *parent=0, const char *name=0);
	~hexViewEdit();
private:
  int         bytesPerLine; // Anzahl Zeichen pro Zeile
  QChar       byteUnderCursor();
  void        undoAdd();
  void        undoOneItem();
  void        updatePos(uint offset);

  void        setByteUnderCursor(uchar);
  bool        TABFLAG;
  void        moveCursor(int xoff, int yoff);
  void        cursorToNextPos();
  void        cursorBack();
  void        updateCursor();
  void        scroll(int yoff);
  QScrollBar  *scrollbar;
  void        updateInfo();
  QFont       *myFont;
  // Editing
  struct ub   *undoList;
  int         undos;
  bool        digit;
  uchar       digit1,digit2;

  //
  bool        resizing;   // resizeevent darf nicht parallel mehrmals arbeiten!!
  QTimer      *ctimer;
  bool        blinkflag;

  FILE        *file;
  QString     filename;
  // Editor Variablen & Felder
  char        *filebuffer;                            // buffer for reading bytes
  uint        buffersize;                             // size of filebuffer, which will be used actually
  uint        filepos;                                // fileposition, which is shown on first visible charakter on screen
  uint        readed;                                 // if readed != buffersize, this is the last block of file
  uint        filesize;
  uint        fontsize;

  int         lines;                                  // count of lines on screen
  bool        edit;                                   // Flag, if we only show or edit
  QString     byte2String(int hex);                   // convert a int to a 2 digit String
  QString     word2String(int hex);                   // convert a int to a 4 digit String
  QString     long2String(int hex);                   // convert a int to a 8 digit String
  QString     mem2String(int off,char * addr, int len);  // convert n bytes from address to a readable string
  QPixmap     *bufferPixmap;                          // buffered pixmap
  int         cursorX, cursorY;                       // X/Y Position of Cursor on screen, not on filepos!!
  int         cursorIndex;                            // cursorindex ist die Entfernung in BYTES des Cursors vom ersten sichtbaren Zeichen
                                                    // Sprich erste Zeile ist 0000:4000 und CursorX = 5, CursorY = 2
                                                    // dann ist cursorindex = 0x4000 + (cursorY*16) + cursorX

  void        read();                                 // reads <buffersize> Bytes from File to <filebuffer>
  void        fillBuffer();                           // draws the text from read() to the buffer pixmap
  void        deleteCursor();
private slots:
  void        keyPressEvent(QKeyEvent *);
  void        keyReleaseEvent(QKeyEvent *);
  void        mousePressEvent(QMouseEvent *);
  void        mouseReleaseEvent(QMouseEvent *);
  void        resizeEvent(QResizeEvent *e);
  void        paintEvent(QPaintEvent *e);
  void        cursorTimer();
  void        scrollbarChanged(int p);
signals:
  void        info1changed(QString s);
  void        info2changed(QString s);
  void        closeParent();
};

#endif
