/***************************************************************************
                          kciconview.cpp  -  description
                             -------------------
    begin                : Wed Feb 6 2002
    copyright            : (C) 2002 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/


#include "kciconview.h"

#include <qcursor.h>
#include <qtranslator.h>
#include <qaction.h>
#include <qaccel.h>
#include <qpainter.h>
#include <qcolordialog.h>
//****************************************************************************************
#include "kciconviewitem.h"
//****************************************************************************************

kcIconView::kcIconView(QWidget *parent, const char *name ) : QIconView(parent,name)
{
  countReady = true;
  popupMenu = new QPopupMenu(this);
  setFocusPolicy(StrongFocus);
  contextTimer = new QTimer(this);
  connect(contextTimer,SIGNAL(timeout()),SLOT(contextMenuShow()));
  dirOperator = new kcDirOperator;
  dirOperator->setname(name);  // Binding an den ListViewnamen, macht das handling der temporaeren dirs easy !
  setDragAutoScroll(true);
  mousePressed = false;
  button = 0;
  RMBpressedOn = 0L;
  n = name;

  setSelectionMode(QIconView::Multi);
  setAcceptDrops(true);
  clipper()->setAcceptDrops(true);

  QSettings settings;
  bool cfg = settings.readBoolEntry("/KCommander/IconView/ArrangeLR/",true);
  if(cfg) setArrangement(QIconView::LeftToRight);
  else setArrangement(QIconView::TopToBottom);

  cfg = settings.readBoolEntry("/KCommander/IconView/leftSide/",true);
  if(cfg) setItemTextPos(Right);
  else setItemTextPos(Bottom);

  setGridX( settings.readNumEntry(QString("/KCommander/IconView/gridSize/"),200) );                                        // configurable

    setResizeMode( Adjust );
    setWordWrapIconText( FALSE );

  connect( this, SIGNAL( doubleClicked( QIconViewItem * ) ),   SLOT( slotFolderSelected( QIconViewItem * ) ) );
  connect( this, SIGNAL( returnPressed( QIconViewItem * ) ),   SLOT( slotFolderSelected( QIconViewItem * ) ) );

  oldDirectory = "";
//  connect(dirOperator,SIGNAL(gotFileList(QPtrList<struct_fileinfo> *)),   SLOT(gotFileList(QPtrList<struct_fileinfo> *)));
  connect(dirOperator,SIGNAL(gotFile(const struct_fileinfo *)),           SLOT(gotFile(const struct_fileinfo *)));
  connect(dirOperator,SIGNAL(dirReady()),                                 SLOT(dirReady()));
  connect(dirOperator,SIGNAL(dirNotExist()),                              SLOT(dirNotExist()));
  connect(this,SIGNAL(itemRenamed ( QIconViewItem *, const QString &)),SLOT(itemWithNewName(QIconViewItem *, const QString &))) ;

    setVScrollBarMode( Auto );
    setHScrollBarMode( Auto );

    setAutoArrange( TRUE );
    setSorting( TRUE );

  QString n(name);
  if( n == "ListView1")
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView1Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView1Highlight/"), "#ff0000");
  }
  else
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView2Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView2Highlight/"), "#ff0000");
  }
  clipper()->setPaletteBackgroundColor(bg) ;

// config ////////////////////////////////////////////////
  showDotFiles = false;
  if( n == "ListView1")
      showDotFiles = settings.readBoolEntry("/KCommander/General/DontShowDotfilesLeft/", false);
  else
      showDotFiles = settings.readBoolEntry("/KCommander/General/DontShowDotfilesRight/", false);

      itemItalic = settings.readBoolEntry(QString("/KCommander/IconView/italic/"), true);
      itemBold = settings.readBoolEntry(QString("/KCommander/IconView/bold/"), true);
///////////////////////////////////////////////////////////////

  inZipFile = false;
  actItemWidth = 0;
  arrangeItemsInGrid();
}

kcIconView::~kcIconView(){
}

/*
ButtonState QKeyEvent::state () const Returns the keyboard modifier flags that existed immediately before the event occurred.
The returned value is ShiftButton, ControlButton and AltButton OR'ed together.
key()
text()
*/

void kcIconView::keyPressEvent(QKeyEvent *e)
{
  // Folgende Keys vom QListView handeln lassen, und wech hier!
  if(e->key() == Qt::Key_Home ||
     e->key() == Qt::Key_End ||
     e->key() == Qt::Key_Left ||
     e->key() == Qt::Key_Up ||
     e->key() == Qt::Key_Right ||
     e->key() == Qt::Key_Down ||
     e->key() == Qt::Key_PageUp ||
     e->key() == Qt::Key_PageDown ||
     e->key() == Qt::Key_Tab
  )
  {
    QIconView::keyPressEvent(e);
    e->accept();
    refreshInfo();
    return;
  }
  if(  e->key() == Qt::Key_Enter ||
       e->key() == Qt::Key_Return
  )
  {
    // Wenn ENTER ohne Sonderkey, dann zu QListView durchreichen
    if(e->state() == 0)
    {
      QIconView::keyPressEvent(e);
      e->accept();
      return;
    }
    if(e->state() == Qt::AltButton)
    {
      e->accept();
      return;
    }
  }

  // Space selbst benutzen !!
  if(e->key() == Qt::Key_Space)
  {
    QIconView::keyPressEvent(e);

    // jetzt schauen, ob es sich um ein Dir handelt,
    // ggfls recursiv die Bytes auslesen
    // ....
    // danach wech hier
    e->accept();
    refreshInfo();
    return;
  }

  if (e->key() == Key_Plus && (e->state() & ControlButton))
  {
    slotCTRL_PlusPressed();
    e->accept();
    return;
  }
  if(e->state() == 0 || e->state() == ShiftButton)
  {
    if(e->text())
    {
      emit activateCommandBox(e->text());
      e->accept();
      return;
    }
  }
  if (e->key() == Key_Minus && (e->state() & ControlButton))
  {
    slotCTRL_MinusPressed();
    e->accept();
    return;
  }
  if (e->key() == 42 && e->state() == 0x4000) // Taste '*'
  {
    slotInvertSelection();
    e->accept();
    refreshInfo();
    return;
  }

  e->ignore();
}

//*****************************************************************
//*****************************************************************
void kcIconView::setDir(QString dir)
{
  setCursor(QCursor(WaitCursor));
  emit pathButtonTextChanged("Reading Dir - wait");
//  oldDirectory = directory;
  directory = dir;
  if(dir.right(1) != "/")
    directory += "/";
  clear();
  // ok, now we call the Diroperator with argument dir
  qApp->processEvents(500);
  dirOperator->readDir(dir);
  emit pathButtonTextChanged(dir);
}
void kcIconView::dirNotExist()
{
  if(oldDirectory == "")
    setDir("/");
  else
    setDir(oldDirectory);
}
void kcIconView::dirReady()
{
  // If dir was readed, dirReady will be emitted
  emit pathButtonTextChanged(directory);
  if(oldDirectory != "")
  {
    oldDirectory = oldDirectory.left(oldDirectory.length()-1);
    QIconViewItem *item =  findItem(oldDirectory.right(oldDirectory.length()-1 - oldDirectory.findRev('/')),0);
    setCurrentItem(item);
  }
  else
  {
    setCurrentItem(firstItem());
  }
  // write this Dirname to config
  QSettings settings;
  QString s;
  QString n(name());
  if(n == "ListView1")
    s = "Left Dir";
  else
    s = "Right Dir";
  settings.writeEntry("/KCommander/Defaults/" + s, directory);
  emit dirChanged(directory);
  setCursor(ArrowCursor);
}

void kcIconView::gotFileList(QPtrList<struct_fileinfo> *files)
{
}
void kcIconView::gotFile(const struct_fileinfo *file)
{
  if(showDotFiles && file->_fileName.left(1) == "." && file->_fileName.left(2) != ".." ) return;
  new kcIconViewItem( file, this);
}


void kcIconView::slotFolderSelected(QIconViewItem *i)
{
  kcIconViewItem *kfli = (kcIconViewItem *)i;
  if(kfli->isFile() && kfli->isReadable())
  {
    QFileInfo inf(directory+kfli->text());
    if( inf.extension(false) == "gz" ||
        inf.extension(false) == "tgz" ||
        inf.extension(false) == "rar" ||
        inf.extension(false) == "ace" ||
        inf.extension(false) == "zip")
    {
//      clear();
      setDir(directory+kfli->text());
    }
    else
    {
      //mal sehen ob man es ausfuehren kann also testen mit QProcess start
      // wenn es geht das ist es gut wenn nicht dann auch
      if(! startProgram(inf))
      {
        // dann muss man eben sehen was es ist fuer ein File
      }
    }
  }

  else if(kfli->isDir() && kfli->isReadable())
  {
    QString newDir;
    if(kfli->text() == "..")      // cd UP
    {
      oldDirectory = directory;
      newDir = directory.left(directory.findRev('/',-2)+1);
    }
    else
    {
      oldDirectory = "";
      newDir = directory + kfli->text();
    }
    clear(); //Saubermachen !
    setDir(newDir);
  }
}

bool  kcIconView::startProgram(QFileInfo fi)
{
  QProcess proc;
  proc.addArgument(fi.fileName());
  return proc.start();
}

// DND start **************************************************************************************
void kcIconView::contentsDragEnterEvent( QDragEnterEvent *e )
{
    if ( !QUriDrag::canDecode(e) ) {
        e->ignore();
        return;
    }
    setFocus();
    oldCurrent = currentItem();
    QIconViewItem *i = findItem( contentsToViewport(e->pos()) );
    if ( i )
    {
        dropItem = i;
    }
}


void kcIconView::contentsDragMoveEvent( QDragMoveEvent *e )
{
    // tritt auf, wenn ein DragObject ber unser Widget wandert
    if ( !QUriDrag::canDecode(e) )
    {
        e->ignore();
        return;
    }

    QPoint vp = contentsToViewport( ( (QDragMoveEvent*)e )->pos() );
    kcIconViewItem *i = (kcIconViewItem *)findItem( vp );
    if(i)
    {
      // das Item unter dem Dragobject wird markiert, um sichtbar zu machen, wo mann fallen lsst
      // und diese Kackroutine ist sooooooooo langsaaaammmm
      setCurrentItem(i);
      // in Statuszeile Ziel anzeigen
      refreshInfo();
      if(dragItem == i)
      {
          dropItem = i;
          // wenn drag auf's selbe Item, dann wech hier
          e->ignore();
          return;
      }
      else
        e->accept();
    }
    else
    {
        e->accept();
//        dropItem = 0;
    }
}

void kcIconView::contentsDragLeaveEvent( QDragLeaveEvent * )
{
    dropItem = 0;
    refreshInfo();
}

void kcIconView::contentsDropEvent( QDropEvent *e )
{
    if ( !QUriDrag::canDecode(e) ) {
        e->ignore();
        return;
    }

    QIconViewItem *item = findItem(e->pos());
    if ( item )
    {
      QStrList lst;
      QUriDrag::decode( e, lst );
      QString str;
      // Hier nen bischen einfgen....
      e->accept();
      str = "COPY\n";
      for ( uint i = 0; i < lst.count(); ++i )
      {
          QString filename = lst.at( i );
          str += filename + "\n";
      }
      str += QString( "TO\n %1" ).arg( fullPath(item) );
      dlgCopy *cp = new dlgCopy(lst,fullPath(item),this);
      cp->exec();
//      QMessageBox::information( this, "Drop target", str, "OK" );
    }
    else
    {
      QStrList lst;
      QUriDrag::decode( e, lst );
      QString str;
      // Hier nen bischen einfgen....
      e->accept();
      str = "COPY\n";
      for ( uint i = 0; i < lst.count(); ++i )
      {
          QString filename = lst.at( i );
          str += filename + "\n";
      }
      str += QString( "TO\n %1" ).arg( directory );
      dlgCopy *cp = new dlgCopy(lst,directory,this);
      cp->exec();
//      QMessageBox::information( this, "Drop target", str, "OK" );
    }
}
// END DND **************************************************************************************
QString kcIconView::fullPath(QIconViewItem* item)
{
    QString fullpath = dirname() + item->text();
    return fullpath;
}
void kcIconView::contentsContextMenuEvent ( QContextMenuEvent * e )
{
}
void kcIconView::contentsMousePressEvent( QMouseEvent* e )
{
  mouseX = e->globalX()+5;
  mouseY = e->globalY()+5;
  // Auf jeden Fall erstmal das Item unter Mouse berechnen
  // brauchen wir hier nicht //
  QIconViewItem *i = findItem( e->pos() );
  ensureItemVisible(i);
  // den gedrckten Button merken
  button = e->button();
  // nun verzweigen:
  // Links DND
  // rechts markieren
  RMBpressedOn = 0L;
  switch(button)
  {
    case LeftButton:
      if(i)
      {
        setCurrentItem(i);
        refreshInfo();
      }
      break;
    case RightButton:
      if(i)
      {
        RMBpressedOn = i;
        if(i->isSelected())
          mark = false;
        else
          mark = true;
        setCurrentItem(i);
        setSelected(i,mark);
        refreshInfo();
        firstSelectedItem = i;
      }
      contextTimer->start(1200,true);
      break;
    case MidButton:
      break;
  }
  mousePressed = true;
//  QListView::contentsMousePressEvent(e);
}

void kcIconView::contentsMouseMoveEvent( QMouseEvent* e )
{
    // Diese Routine nur durchlaufen, wenn Mouse gedrckt wurde
    if(!mousePressed)
      return;
//    QIconViewItem *item = findItem( presspos );
    dragItem = 0;
    switch(button)
    {
      case LeftButton:
      {
        if ( mousePressed && ( presspos - e->pos() ).manhattanLength() > QApplication::startDragDistance() )
        {
            QIconViewItem *item = findItem( e->pos() );
            mousePressed = FALSE;
            if ( item )
            {
                dragItem = (kcIconViewItem *)item;
                QStringList fnames;
                bool counter = false;
                QIconViewItem *it = firstItem();
                kcIconViewItem * ki;
            do
            {
              ki = (kcIconViewItem *)it;
              if ( ki->isSelected() )
                      {
                        counter = true;
                        fnames.append(fullPath(ki));
                      }
                      it = it->nextItem();
            }
            while( it ) ;

                if(!counter)
                {
                  fnames.append(fullPath(currentItem()));
                }
                QString source = fullPath(item);
                QUriDrag* ud = new QUriDrag(viewport());
                ud->setUnicodeUris( fnames );

                ud->setPixmap(*item->pixmap());

                ud->dragCopy();
        }
        }
        break;
      }
      case RightButton:
      {
          QIconViewItem *i2 = findItem( e->pos() );
          setSelected(i2,mark);
          setCurrentItem(i2);
          if(i2 != firstSelectedItem)
            contextTimer->stop();
          refreshInfo();
          break;
      }
      default:
//          mousePressed = FALSE;
      break;
    }
}

void kcIconView::contentsMouseReleaseEvent( QMouseEvent *e )
{
    mousePressed = FALSE;
	  button = e->button();
	  switch(button)
	  {
	  	case LeftButton:
	  	{
	  		clearSelection();
	  		break;
	  	}
	  	case RightButton:
	  	{
				contextTimer->stop();
	  		break;
	  	}
	  	case MidButton:
	  	{
	  		contextMenuShow();
	  		break;
	  	}
	  }
    button = 0;
    dragItem = 0;
//    QListView::contentsMouseReleaseEvent(e);
}
void kcIconView::contextMenuShow()
{
  popupMenu->clear();

  QAction *chFileProp = new QAction(tr("Change properties of file"),kcFileIconProvider::getAppPixmap("config"), tr("Change &properties for file"), QAccel::stringToKey(tr("P")), this);
  chFileProp->setWhatsThis(tr("See and change the properties for a file"));
  connect(chFileProp,SIGNAL(activated()),this,SLOT(makePropertiesForFileChange()));

  QAction *mkSymlink = new QAction(tr("Make symlink"),kcFileIconProvider::getAppPixmap("symlink"), tr("Make &symlink"), QAccel::stringToKey(tr("S")), this);
  mkSymlink->setWhatsThis(tr("Make a new symbolic link"));
  connect(mkSymlink,SIGNAL(activated()),this,SLOT(makeSymlink()));

  QAction *toggleShowDot = new QAction(tr("Show or hide hidden files"),tr("Show &hidden files"), QAccel::stringToKey(tr("H")), this,"showdotfiles",showDotFiles);
  toggleShowDot->setToggleAction(true);
  toggleShowDot->setOn(! showDotFiles);
  toggleShowDot->setWhatsThis(tr("Click here to show hidden files or hide them"));
  connect(toggleShowDot,SIGNAL(activated()),this,SLOT(setShowDotFiles()));

  QAction *fileRename = new QAction(tr("Rename the file"),kcFileIconProvider::getAppPixmap("rename"), tr("&Rename this file"), QAccel::stringToKey(tr("R")), this);
  fileRename->setWhatsThis(tr("Give the file another name"));
  connect(fileRename,SIGNAL(activated()),this,SLOT(renameFile()));

  QAction *chView = new QAction(tr("Change to ListView"),kcFileIconProvider::getPixmap("listView"), tr("Change to List&View"), QAccel::stringToKey(tr("V")), this);
  chView->setWhatsThis(tr("Change the view to Listview"));
  connect(chView,SIGNAL(activated()),this,SLOT(makeViewChange()));

  QAction *newListViewColor = new QAction(tr("Change backgroundcolor"),kcFileIconProvider::getAppPixmap("paint"), tr("&Change backgroundcolor"), QAccel::stringToKey(tr("C")), this);
  newListViewColor->setWhatsThis(tr("Here you can change the background of the views"));
  connect(newListViewColor,SIGNAL(activated()),this,SLOT(getColorDialog()));

  QAction *fileMail = new QAction(tr("Send per mail"),kcFileIconProvider::getAppPixmap("mail"), tr("&Send per Mail"), QAccel::stringToKey(tr("S")), this);
  newListViewColor->setWhatsThis(tr("Here you can send current file as mail"));
  connect(fileMail,SIGNAL(activated()),this,SLOT(sendFileAsMail()));

  if(RMBpressedOn)
    chFileProp->addTo(popupMenu);
  if(RMBpressedOn)
    fileRename->addTo(popupMenu);
  if(RMBpressedOn)
    mkSymlink->addTo(popupMenu);
  popupMenu->insertSeparator();
  chView->addTo(popupMenu);
  newListViewColor->addTo(popupMenu);
  popupMenu->insertSeparator();
  toggleShowDot->addTo(popupMenu);
  popupMenu->insertSeparator();
  if(RMBpressedOn)
    fileMail->addTo(popupMenu);
  mousePressed = false;
  popupMenu->exec(QPoint(mouseX, mouseY));
}
void kcIconView::refreshInfo()
{
  emit selectionChanged();
}
void kcIconView::focusInEvent(QFocusEvent *e)
{
  emit focusIn();
}
void kcIconView::focusOutEvent(QFocusEvent *e)
{
  emit focusOut();
}
void kcIconView::sendKey(QKeyEvent *e)
{
  keyPressEvent(e);
  setFocus();
}

void  kcIconView::getColorDialog()
{
  QColor nc = QColorDialog::getColor();
  // If the user dont pressed 'CANCEL' then
  if(nc.isValid())
  {
     bg = nc; // oldColor = newColor

     QSettings settings;
     if(QString(name()) == "ListView1")
       settings.writeEntry(QString("/KCommander/Colors/ListView1Background/"), bg.name());
     else
       settings.writeEntry(QString("/KCommander/Colors/ListView2Background/"), bg.name());

     clipper()->setPaletteBackgroundColor(bg) ;
     setBackgroundColor(bg) ;
     repaint();
  }
}

QColor  kcIconView::getBackgroundColor()
{
  return bg;
}
QColor  kcIconView::getHighlightColor()
{
  return hl;
}

QString kcIconView::dirname()
{
  return directory;
}

void	kcIconView::reRead()
{
  clear();
  dirOperator->reRead();
}

void kcIconView::slotCTRL_MinusPressed()
{
    selectAll(false);
    refreshInfo();
}
void kcIconView::slotCTRL_PlusPressed()
{
    selectAll(true);
    QIconViewItem *i = findItem("..");
    if(i)
      i->setSelected(false);
    refreshInfo();
}

void kcIconView::slotMinusPressed()
{
    QString expression = getExpression();
    if(expression == "")
      return;
    else
      setSelection(expression, false);
    refreshInfo();
}
void kcIconView::slotPlusPressed()
{
    QString expression = getExpression();
    if(expression == "")
      return;
    else
      setSelection(expression, true);

    refreshInfo();
}
void kcIconView::setSelection(QString exp, bool set)
{
  QDir dir(dirname());
  dir.setNameFilter(exp);
  dir.setFilter( QDir::Files | QDir::Hidden | QDir::Dirs );

  const QFileInfoList *list = dir.entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;

  while ( (fi = it.current()) != 0 )
  {
    QIconViewItem *i = findItem(fi->fileName().latin1());
    if(i)
    {
      setSelected(i,set);
    }
     ++it;
  }
  QIconViewItem *i = findItem("..");
  if(i)
    setSelected(i,false);
}

QString kcIconView::getExpression()
{
  dlgSelect *dlg = new dlgSelect();
  dlg->exec();
  if(!dlg->result())
    return "";
  QString s = dlg->resultString();
  delete dlg;
  return strdup(s);
}

void kcIconView::makeViewChange()
{
  emit(changeViewer(name()));
}

void  kcIconView::makePropertiesForFileChange()
{
  if(!currentItem())
    return;
  if(this->currentItem()->text() == "..")
    return;
  emit(changePropertiesForFile(QFileInfo(directory+this->currentItem()->text())));
}

void  kcIconView::renameFile()
{
  if(!currentItem())
    return;
  if(this->currentItem()->text() == "..")
    return;
  renFileInfo = QFileInfo(directory + currentItem()->text());
  this->currentItem()->rename();
}
void kcIconView::itemWithNewName(QIconViewItem * item, const QString & name )
{
//#error Arno, er bekommt hier falsche Werte, im ListView geht es aber
//#error der neue Name ist Faslch
  QString oldName = directory + name;
  QString newName = renFileInfo.absFilePath();
  debug("Itemtext = " + item->text());
  debug("Alter Name = " + oldName);
  debug("Neuer Name = " + newName);
  debug("name = " + name);

//  system(QString("mv "+ renFileInfo.absFilePath() + " "+directory+name));
  // ist unbedingt noch auszuwerten !!!!!!!!!!
//  reRead();
}

void  kcIconView::setShowDotFiles()
{
  QSettings settings;
  showDotFiles = !showDotFiles;
  QString n(name());
  if( n == "ListView1")
      settings.writeEntry("/KCommander/General/DontShowDotfilesLeft/", showDotFiles );
  else
      settings.writeEntry("/KCommander/General/DontShowDotfilesRight/", showDotFiles);
  reRead();
}

void kcIconView::slotInvertSelection()
{
   kcIconViewItem *kiv;
   kiv = (kcIconViewItem*)firstItem();
   // iterate through all items of the list view
   while(kiv)
   {
     if(kiv->text() != "..")
     {
       if ( kiv->isSelected() )
        kiv->setSelected(false);
       else
        kiv->setSelected(true);
     }
     kiv->repaint();
     kiv = (kcIconViewItem*)kiv->nextItem();
   }
}

void kcIconView::makeSymlink()
{
  if(!currentItem())
    return;
  if(this->currentItem()->text() == "..")
    return;
  emit(makeSymlinkToFile(QFileInfo(directory+this->currentItem()->text())));
}

void kcIconView::settingsChanged()
{
  QSettings settings;
  bool cfg = settings.readBoolEntry("/KCommander/IconView/ArrangeLR/",true);
  if(cfg) setArrangement(QIconView::LeftToRight);
  else setArrangement(QIconView::TopToBottom);

  cfg = settings.readBoolEntry("/KCommander/IconView/leftSide/",true);
  if(cfg) setItemTextPos(Right);
  else setItemTextPos(Bottom);

  setGridX( settings.readNumEntry(QString("/KCommander/IconView/gridSize/"),200) );                                        // configurable

  QString n(name());
  if( n == "ListView1")
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView1Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView1Highlight/"), "#ff0000");
  }
  else
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView2Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView2Highlight/"), "#ff0000");
  }
  clipper()->setPaletteBackgroundColor(bg) ;

  itemItalic = settings.readBoolEntry(QString("/KCommander/IconView/italic/"), true);
  itemBold = settings.readBoolEntry(QString("/KCommander/IconView/bold/"), true);
}

void kcIconView::sendFileAsMail()
{
  if(!currentItem())
    return;
  if(this->currentItem()->text() == "..")
    return;

    QSettings settings;
    QString ask = settings.readEntry("/KCommander/Mail/Mailusername","none");
    if(ask == "none")
    {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("No valid username given.\nTake a look at the optionsdialog \nand make the right informations please."), 0, 0 );
    	return;
    }
    QString ask1 = settings.readEntry("/KCommander/Mail/Mailserver","none");
    if(ask1 == "none")
    {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("No valid mailserver given.\nTake a look at the optionsdialog \nand make the right informations please."), 0, 0 );
    	return;
    }

  QFileInfo inf(directory+this->currentItem()->text());

   if(inf.isDir())
   {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("It is not yet possible to send folders.\n\nHint: Make an archive and send this one.\n"), 0, 0 );
    	return;
   }

  mailComposer *c = new mailComposer(inf);
  c->from->setText(ask);    // mit der Mailingliste geht das irgendwie noch nicht :(
  c->from->setEnabled(false);
  c->resize(500,300);
  c->show();
}
