/***************************************************************************
                          kclistview.cpp  -  description
                             -------------------
    begin                : Thu Dec 13 2001
    copyright            : (C) 2001 by Christian Fricke & Ren Mrten
    email                : cfricke@kcommander.org
 ***************************************************************************/
#include <stdlib.h>

#include <qcursor.h>
#include "kclistview.h"

static const char *rwx[] = {"--- ","--x ","-w- ","-wx ","r-- ","r-x ","rw- ","rwx " };

//*****************************************************************************************
#include "kclistviewitem.h"
//*****************************************************************************************
kcListView::kcListView(QWidget *parent, const char *name ) : QListView(parent,name)
{
  countReady = true;
  header()->setMaximumHeight(25);
  itemLightColor = false;

  popupMenu = new QPopupMenu(this);
  setFocusPolicy(StrongFocus);

  contextTimer = new QTimer(this);
  connect(contextTimer,SIGNAL(timeout()),SLOT(contextMenuShow()));

  dirOperator = new kcDirOperator;
  dirOperator->setname(name);  // Binding an den ListViewnamen, macht das handling der temporaeren dirs easy !
  setDragAutoScroll(true);
  mousePressed = false;
  button = 0;
  RMBpressedOn = 0L;
  n = name;
  setAcceptDrops(true);
  viewport()->setAcceptDrops( TRUE );
  setAllColumnsShowFocus(true);
  setMultiSelection(true);
  setShowSortIndicator(true);

  addColumn( tr("Fileame" ));
  addColumn( tr("Size" ));
  addColumn( tr("Date" ));
  addColumn( tr("Permissions" ));
  addColumn( tr("Owner" ));
  addColumn( tr("Group" ));

  setColumnAlignment(0, Qt::AlignLeft);
  setColumnAlignment(1, Qt::AlignRight);
  setColumnAlignment(2, Qt::AlignRight);
  setColumnAlignment(3, Qt::AlignRight);
  setColumnAlignment(4, Qt::AlignRight);
  setColumnAlignment(5, Qt::AlignRight);

  connect( this, SIGNAL( doubleClicked( QListViewItem * ) ),   SLOT( slotFolderSelected( QListViewItem * ) ) );
  connect( this, SIGNAL( returnPressed( QListViewItem * ) ),   SLOT( slotFolderSelected( QListViewItem * ) ) );

  oldDirectory = "";
  connect(dirOperator,SIGNAL(gotFileList(QPtrList<struct_fileinfo> *)),   SLOT(gotFileList(QPtrList<struct_fileinfo> *)));
  connect(dirOperator,SIGNAL(gotFile(const struct_fileinfo *)),           SLOT(gotFile(const struct_fileinfo *)));
  connect(dirOperator,SIGNAL(dirReady()),                                 SLOT(dirReady()));
  connect(dirOperator,SIGNAL(dirNotExist()),                              SLOT(dirNotExist()));
  connect(this,SIGNAL(itemRenamed ( QListViewItem *,int, const QString &)),SLOT(itemWithNewName(QListViewItem *,int, const QString &))) ;

  QSettings settings;
  QString n(name);
  if( n == "ListView1")
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView1Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView1Highlight/"), "#ff0000");
  }
  else
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView2Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView2Highlight/"), "#ff0000");
  }
  clipper()->setPaletteBackgroundColor(bg) ;
  setBackgroundColor(bg) ;

// config ////////////////////////////////////////////////
  QString s;
  showDotFiles = false;
  if( n == "ListView1")
      showDotFiles = settings.readBoolEntry("/KCommander/General/DontShowDotfilesLeft/", false);
  else
      showDotFiles = settings.readBoolEntry("/KCommander/General/DontShowDotfilesRight/", false);

      itemItalic = settings.readBoolEntry(QString("/KCommander/ListView/italic/"), true);
      itemBold = settings.readBoolEntry(QString("/KCommander/ListView/bold/"), true);
      recolor = settings.readBoolEntry(QString("/KCommander/ListView/recolor/"), true);
///////////////////////////////////////////////////////////////

  inZipFile = false;
}
kcListView::~kcListView(){
}

void kcListView::keyPressEvent(QKeyEvent *e)
//void kcListView::keyReleaseEvent(QKeyEvent *e)
{
//  debug("KEY: %d ", e->key());
//  debug("STATE: %d ", e->state());
  if(!countReady)
  {
    e->accept();
    return;
  }


  // Folgende Keys vom QListView handeln lassen, und wech hier!
  if(e->key() == Qt::Key_Home ||
     e->key() == Qt::Key_End ||
     e->key() == Qt::Key_Left ||
     e->key() == Qt::Key_Up ||
     e->key() == Qt::Key_Right ||
     e->key() == Qt::Key_Down ||
     e->key() == Qt::Key_PageUp ||
     e->key() == Qt::Key_PageDown ||
     e->key() == Qt::Key_Tab
  )
  {
    QListView::keyPressEvent(e);
//    qApp->processEvents(100);
    refreshInfo();
    e->accept();
    return;
  }
//*********************************************************************************************
  if(e->key() == Qt::Key_Insert)
  {
    // EINFG Taste = SPACE und eins Runter
    QKeyEvent *ee;

    // 1) EINF wurde gedrckt
    // 2) wenn text(0) == ".." dann nur eine Zeile runter, OHNE SPACE (weiter bei 10)  )
    // ansonsten
    if(currentItem()->text(0) != tr(".."))
    {
      // a) Taste annehmen
      e->accept();
      // b) SPACE drcken
      if(currentItem()->isSelected())
        currentItem()->setSelected(false);
      else
        currentItem()->setSelected(true);
      currentItem()->repaint();
      // c) handelt es sich um ein Dir ??
      // ja, dann Bytes zhlen
      if(currentItem()->text(1) == tr("Directory") && currentItem()->isSelected() == true )
      {
         proc = new QProcess( this );
         proc->addArgument( "du" );
         proc->addArgument( "-b" );
         proc->addArgument( "-s" );
         proc->addArgument(fullPath(currentItem()));
         connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()) );
         proc->setCommunication(QProcess::Stdin | QProcess::Stdout | QProcess::Stderr);
         countReady = false;
         proc->start();
         // nun solange warten, bis der counter zurckkehrt...
         while(!countReady)
         {
           qApp->processEvents();
         }
      }
    }
    // auf jeden Fall CursorDown Emulieren
    ee = new QKeyEvent(QEvent::KeyPress, 4117, 0, 0, 0);
    QListView::keyPressEvent(ee);
    refreshInfo();
  }
//*********************************************************************************************
  if(  e->key() == Qt::Key_Enter ||
       e->key() == Qt::Key_Return
  )
  {
    // Wenn ENTER ohne Sonderkey, dann zu QListView durchreichen
    if(e->state() == 0)
    {
      QListView::keyPressEvent(e);
      e->accept();
      return;
    }
    if(e->state() == Qt::AltButton)
    {
      e->accept();
      return;
    }
  }
//*********************************************************************************************
  // Space selbst benutzen !!
  if(e->key() == Qt::Key_Space)
  {
    if(currentItem()->text(0) == tr(".."))
    {
      e->accept();
      return;
    }
    QListView::keyPressEvent(e);
    qApp->processEvents(500);
    // jetzt schauen, ob es sich um ein Dir handelt,
    // ggfls recursiv die Bytes auslesen
    // ....
    // danach wech hier
    if(currentItem()->text(1) == tr("Directory") &&
      currentItem()->isSelected() == true
    )
    {
       proc = new QProcess( this );
       proc->addArgument( "du" );
       proc->addArgument( "-b" );
       proc->addArgument( "-s" );
       proc->addArgument(fullPath(currentItem()));
       connect( proc, SIGNAL(readyReadStdout()), this, SLOT(readFromStdout()) );
       proc->setCommunication(QProcess::Stdin | QProcess::Stdout | QProcess::Stderr);
       proc->start();
      // nun solange warten, bis der counter zurckkehrt...
      countReady = false;
      while(!countReady)
      {
        qApp->processEvents();
      }
    }
    else
      refreshInfo();

    e->accept();
    return;
  }
//*********************************************************************************************
  if (e->key() == Key_Plus && (e->state() & ControlButton))
  {
    slotCTRL_PlusPressed();
    e->accept();
    return;
  }
//*********************************************************************************************
  if(e->state() == 0 || e->state() == ShiftButton)
  {
    if(e->text())
    {
      emit activateCommandBox(e->text());
      e->accept();
      return;
    }
  }
//*********************************************************************************************
  if (e->key() == Key_Minus && (e->state() & ControlButton))
  {
    slotCTRL_MinusPressed();
    e->accept();
    return;
  }
//*********************************************************************************************
  if (e->key() == 42 && e->state() == 0x4000) // Taste '*'
  {
    slotInvertSelection();
    e->accept();
    refreshInfo();
    return;
  }

  e->ignore();
}
void kcListView::readFromStdout()
{
  QString s = proc->readLineStdout();
  if(s.find("du:") != -1)
    return;
  s = s.left(s.find("\t"));
  currentItem()->setText(1,s);
  refreshInfo();
  countReady = true;
}
//*****************************************************************
void kcListView::setDir(QString dir)
{
  setCursor(QCursor(WaitCursor));
  directory = dir;
  if(dir.right(1) != "/")
    directory += "/";
  clear();
  emit pathButtonTextChanged("Reading Dir - wait");
  // ok, now we call the Diroperator with argument dir
  qApp->processEvents(500);
  dirOperator->readDir(dir);
}
void kcListView::dirNotExist()
{
  if(oldDirectory == "")
    setDir("/");
  else
    setDir(oldDirectory);
}
void kcListView::dirReady()
{
  // If dir was readed, dirReady will be emitted
  emit pathButtonTextChanged(directory);

  if(oldDirectory.length() != 0)
  {
    oldDirectory = oldDirectory.left(oldDirectory.length()-1); // SLASH weg
    QListViewItem *item =  findItem(oldDirectory.right(oldDirectory.length()-1 - oldDirectory.findRev('/')),0);
    setCurrentItem(item);
  }
  else
  {
    setCurrentItem(firstChild());
  }

  // write this Dirname to config
  QSettings settings;
  QString s;
  QString n(name());
  if(n == "ListView1")
    s = "Left Dir";
  else
    s = "Right Dir";
  settings.writeEntry("/KCommander/Defaults/" + s, directory);
  emit dirChanged(directory);
  setCursor(ArrowCursor);
}

void kcListView::gotFileList(QPtrList<struct_fileinfo> *files)
{
}
void kcListView::gotFile(const struct_fileinfo *file)
{
  if( showDotFiles && file->_fileName.left(1) == "."
        && file->_fileName.left(2) != ".." )
        return;
  new kcListViewItem( file, this, file->_fileName);
}


void kcListView::slotFolderSelected(QListViewItem *i)
{
  kcListViewItem *kfli = (kcListViewItem *)i;
  if(kfli->isFile() && kfli->isReadable())
  {
    QFileInfo inf(directory+kfli->text(0));
    if( inf.extension(false) == "gz" ||
        inf.extension(false) == "tgz" ||
        inf.extension(false) == "rar" ||
        inf.extension(false) == "ace" ||
        inf.extension(false) == "zip")
    {
//      clear();
      setDir(directory+kfli->text(0));
    }
    else
    {
      //mal sehen ob man es ausfuehren kann also testen mit QProcess start
      // wenn es geht das ist es gut wenn nicht dann auch
      if(! startProgram(inf))
      {
        // dann muss man eben sehen was es ist fuer ein File
      }
    }
  }

  else if(kfli->isDir() && kfli->isReadable())
  {
    QString newDir;
    if(kfli->text(0) == "..")      // cd UP
    {
      oldDirectory = directory;
      newDir = directory.left(directory.findRev('/',-2)+1);
    }
    else
    {
      oldDirectory = "";
      newDir = directory + kfli->text(0);
    }
    clear();
    setDir(newDir);
  }
}

bool  kcListView::startProgram(QFileInfo fi)
{
  QProcess proc;
  proc.addArgument(fi.fileName());
  return proc.start();
}


void kcListView::contentsDragEnterEvent( QDragEnterEvent *e )
{
    // wird ein Object in diesen Listview gefhrt, so erhlt dieser den Focus
    setFocus();
    e->accept();
    return;
    // tritt auf, wenn ein DragObject in unser Widget eindringt
    if ( !QUriDrag::canDecode(e) )
    {
        e->ignore();
        return;
    }
    // aktuell markiertes Item merken
    oldCurrent = currentItem();
    // Item unter Maus holen
    QListViewItem *i = itemAt( contentsToViewport(e->pos()) );
    // ist da berhaupt eins?
    if ( i )
    {
        // ja, also dieses fr Drop markieren
      dropItem = i;
    }
}


void kcListView::contentsDragMoveEvent( QDragMoveEvent *e )
{
    // tritt auf, wenn ein DragObject ber unser Widget wandert
    if ( !QUriDrag::canDecode(e) )
    {
        e->ignore();
        return;
    }

    QPoint vp = contentsToViewport( ( (QDragMoveEvent*)e )->pos() );
    kcListViewItem *i = (kcListViewItem *)itemAt( vp );
    if(i)
    {
      // das Item unter dem Dragobject wird markiert, um sichtbar zu machen, wo mann fallen lsst
      // und diese Kackroutine ist sooooooooo langsaaaammmm
      setCurrentItem(i);
      // in Statuszeile Ziel anzeigen
      refreshInfo();
      if(dragItem == i)
      {
          dropItem = i;
          // wenn drag auf's selbe Item, dann wech hier
          e->ignore();
          return;
      }
      else
        e->accept();
    }
    else
    {
        e->ignore();
        dropItem = 0;
    }
}

void kcListView::contentsDragLeaveEvent( QDragLeaveEvent *e )
{
    dropItem = 0;

//    setCurrentItem( oldCurrent );
//    setSelected( oldCurrent, TRUE );
    refreshInfo();
}

void kcListView::contentsDropEvent( QDropEvent *e )
{
    if ( !QUriDrag::canDecode(e) ) {
        e->ignore();
        return;
    }

    QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
    if ( item )
    {
      QStrList lst;
      QUriDrag::decode( e, lst );
      QString str;
      // Hier nen bischen einfgen....
      e->accept();
      str = "COPY\n";
      for ( uint i = 0; i < lst.count(); ++i )
      {
          QString filename = lst.at( i );
          if(filename.left(5) == "file:")
          {
//             str += QString( "   %1\n" ).arg( lst.at( i ) );
//             str += QString( "\n"
//                             "To\n\n"
//                             "   %1" ).arg( filename() );
          }
          str += filename + "\n";
          debug("FILE to copy: " + filename);
          debug(filename.fromUtf8(filename));
      }
      str += QString( "TO\n %1" ).arg( fullPath(item) );
      dlgCopy *cp = new dlgCopy(lst,fullPath(item),this);
      cp->exec();
//      QMessageBox::information( this, "Drop target", str, "OK" );
    }
    else
        e->ignore();
}

QString kcListView::fullPath(QListViewItem* item)
{
    QString fullpath = dirname() + item->text(0);
    return strdup(fullpath);
}
void kcListView::contentsContextMenuEvent ( QContextMenuEvent * e )
{
}
void kcListView::contentsMousePressEvent( QMouseEvent* e )
{
  mouseX = e->globalX()+5;
  mouseY = e->globalY()+5;
  // Auf jeden Fall erstmal das Item unter Mouse berechnen
  QPoint p( contentsToViewport( e->pos() ) );
  QListViewItem *i = itemAt( p );
  // den gedrckten Button merken
  button = e->button();
  // nun verzweigen:
  // Links DND
  // rechts markieren
  RMBpressedOn = 0L;
  switch(button)
  {
    case LeftButton:
      if(i)
      {
        setCurrentItem(i);
        refreshInfo();
      }
      break;
    case RightButton:
      if(i)
      {
        RMBpressedOn = i;
        if(i->text(0) == "..")
          break;
        if(i->isSelected())
          mark = false;
        else
          mark = true;
        setCurrentItem(i);
        setSelected(i,mark);

        refreshInfo();
        firstSelectedItem = i;
      }
      contextTimer->start(1200,true);
      break;
    case MidButton:
      break;
  }
  mousePressed = true;
//  QListView::contentsMousePressEvent(e);
}

void kcListView::contentsMouseMoveEvent( QMouseEvent* e )
{
    // Diese Routine nur durchlaufen, wenn Mouse gedrckt wurde
    if(!mousePressed)
      return;
//    QListViewItem *i = itemAt( contentsToViewport(presspos) );
    dragItem = 0;
    switch(button)
    {
      case LeftButton:
      {
        if ( mousePressed && ( presspos - e->pos() ).manhattanLength() > QApplication::startDragDistance() )
        {
            QListViewItem *item = itemAt( contentsToViewport(e->pos()) );
            mousePressed = FALSE;
            if ( item )
            {
                dragItem = (kcListViewItem *)item;
                QStringList fnames;
                // Create an iterator and give the list view as argument
                QListViewItemIterator it( this );
                // iterate through all items of the list view
                bool counter = false;
                for ( ; it.current(); ++it )
                {
                  if ( it.current()->isSelected() )
                  {
                    counter = true;
                    fnames.append(fullPath(it.current()));
                  }
                }
                if(!counter)
                {
                  fnames.append(fullPath(currentItem()));
                }
                QString source = fullPath(item);
                QUriDrag* ud = new QUriDrag(viewport());
                ud->setUnicodeUris( fnames );
                ud->setPixmap(*item->pixmap(0));
                ud->dragCopy();
            }
        }
        break;
      }
      case RightButton:
      {
          int y = contentsY();
          int h = height();
          QListViewItem *i2 = itemAt( contentsToViewport(e->pos()) );
          if(!i2)
          {
            // hier spter mit timer arbeiten....
            y = e->y() -y;
            if(y < 0)
              scrollBy(0,-5);
            else if(y > h)
              scrollBy(0,5);
            break;
          }
          if(i2->text(0) != "..")
            setSelected(i2,mark);
          setCurrentItem(i2);
          if(i2 != firstSelectedItem)
            contextTimer->stop();
          refreshInfo();
          break;
      }
      default:
//          mousePressed = FALSE;
      break;
    }
}

void kcListView::contentsMouseReleaseEvent( QMouseEvent *e )
{
    mousePressed = FALSE;
	  button = e->button();
	  switch(button)
	  {
	  	case LeftButton:
	  	{
//	  		clearSelection();   // wer hat den Scheiss hier reingeschrieben ?
	  		break;
	  	}
	  	case RightButton:
	  	{
				contextTimer->stop();
	  		break;
	  	}
	  	case MidButton:
	  	{
	  		contextMenuShow();
	  		break;
	  	}
	  }
    button   = 0;
    dragItem = 0;
//    QListView::contentsMouseReleaseEvent(e);
}
void kcListView::contextMenuShow()
{
  popupMenu->clear();

  QAction *chFileProp = new QAction(tr("Change properties of file"),kcFileIconProvider::getAppPixmap("config"), tr("Change &properties for file"), QAccel::stringToKey(tr("P")), this);
  chFileProp->setWhatsThis(tr("See and change the properties for a file"));
  connect(chFileProp,SIGNAL(activated()),this,SLOT(makePropertiesForFileChange()));

  QAction *mkSymlink = new QAction(tr("Make symlink"),kcFileIconProvider::getAppPixmap("symlink"), tr("Make &symlink"), QAccel::stringToKey(tr("S")), this);
  mkSymlink->setWhatsThis(tr("Make a new symbolic link"));
  connect(mkSymlink,SIGNAL(activated()),this,SLOT(makeSymlink()));

  QAction *toggleShowDot = new QAction(tr("Show or hide hidden files"),tr("Show &hidden files"), QAccel::stringToKey(tr("H")), this,"showdotfiles",showDotFiles);
  toggleShowDot->setToggleAction(true);
  toggleShowDot->setOn(! showDotFiles);
  toggleShowDot->setWhatsThis(tr("Click here to show hidden files or hide them"));
  connect(toggleShowDot,SIGNAL(activated()),this,SLOT(setShowDotFiles()));

  QAction *fileRename = new QAction(tr("Rename the file"),kcFileIconProvider::getAppPixmap("rename"), tr("&Rename this file"), QAccel::stringToKey(tr("R")), this);
  chFileProp->setWhatsThis(tr("Give the file another name"));
  connect(fileRename,SIGNAL(activated()),this,SLOT(renameFile()));

  QAction *chView = new QAction(tr("Change to IconView"),kcFileIconProvider::getPixmap("iconView"), tr("Change to Icon&View"), QAccel::stringToKey(tr("V")), this);
  chView->setWhatsThis(tr("Change the view to Iconview"));
  connect(chView,SIGNAL(activated()),this,SLOT(makeViewChange()));

  QAction *newListViewColor = new QAction(tr("Change backgroundcolor"),kcFileIconProvider::getAppPixmap("paint"), tr("&Change backgroundcolor"), QAccel::stringToKey(tr("C")), this);
  newListViewColor->setWhatsThis(tr("Here you can change the background of the views"));
  connect(newListViewColor,SIGNAL(activated()),this,SLOT(getColorDialog()));

  QAction *fileMail = new QAction(tr("Send per mail"),kcFileIconProvider::getAppPixmap("mail"), tr("&Send per Mail"), QAccel::stringToKey(tr("S")), this);
  newListViewColor->setWhatsThis(tr("Here you can send current file as mail"));
  connect(fileMail,SIGNAL(activated()),this,SLOT(sendFileAsMail()));

  if(RMBpressedOn)
    chFileProp->addTo(popupMenu);
  if(RMBpressedOn)
    fileRename->addTo(popupMenu);
  if(RMBpressedOn)
    mkSymlink->addTo(popupMenu);
  popupMenu->insertSeparator();
  chView->addTo(popupMenu);
  newListViewColor->addTo(popupMenu);
  popupMenu->insertSeparator();
  toggleShowDot->addTo(popupMenu);
  popupMenu->insertSeparator();
  if(RMBpressedOn)
    fileMail->addTo(popupMenu);
  mousePressed = false;
  popupMenu->exec(QPoint(mouseX, mouseY));
}
void kcListView::refreshInfo()
{
  emit selectionChanged();
}
void kcListView::focusInEvent(QFocusEvent *e)
{
  emit focusIn();
}
void kcListView::focusOutEvent(QFocusEvent *e)
{
  emit focusOut();
}
void kcListView::sendKey(QKeyEvent *e)
{
  keyPressEvent(e);
  setFocus();
}

void  kcListView::getColorDialog()
{
  QColor nc = QColorDialog::getColor();
  // If the user dont pressed 'CANCEL' then
  if(nc.isValid())
  {
     bg = nc; // oldColor = newColor

     QSettings settings;
     if(QString(name()) == "ListView1")
       settings.writeEntry(QString("/KCommander/Colors/ListView1Background/"), bg.name());
     else
       settings.writeEntry(QString("/KCommander/Colors/ListView2Background/"), bg.name());

     clipper()->setPaletteBackgroundColor(bg) ;
     setBackgroundColor(bg) ;
     repaint();
  }
}

QColor  kcListView::getBackgroundColor()
{
  return bg;
}
QColor  kcListView::getHighlightColor()
{
  return hl;
}

QString kcListView::dirname()
{
    return directory;
}

void	kcListView::reRead()
{
  clear();
  dirOperator->reRead();
}

void kcListView::slotCTRL_MinusPressed()
{
    selectAll(false);
    refreshInfo();
}
void kcListView::slotCTRL_PlusPressed()
{
    selectAll(true);
    QListViewItem *i = findItem("..",0);
    if(i)
      i->setSelected(false);
    refreshInfo();
}
void kcListView::slotMinusPressed()
{
    QString expression = getExpression();
    if(expression == "")
      return;
    else
      setSelection(expression, false);
    refreshInfo();
}
void kcListView::slotPlusPressed()
{
    QString expression = getExpression();
    if(expression == "")
      return;
    else
      setSelection(expression, true);

//    QListViewItem *i = findItem("..",0);
//    if(i)
//      i->setSelected(false);
    refreshInfo();
}


void kcListView::slotInvertSelection()
{
   kcListViewItem *kli;
   QListViewItemIterator it( this );
   // iterate through all items of the list view
   for ( ; it.current(); ++it )
   {
     kli = (kcListViewItem *)it.current();
     if(kli->text(0) != "..")
     {
       if ( kli->isSelected() )
        kli->setSelected(false);
       else
        kli->setSelected(true);
     }
     kli->repaint();
   }
}

void kcListView::setSelection(QString exp, bool set)
{
  QDir dir(dirname());
  dir.setNameFilter(exp);
  dir.setFilter( QDir::Files | QDir::Hidden | QDir::Dirs );

  const QFileInfoList *list = dir.entryInfoList();
  QFileInfoListIterator it( *list );
  QFileInfo *fi;

  while ( (fi = it.current()) != 0 )
  {
    QListViewItem *i = findItem(fi->fileName().latin1(),0);
    if(i)
    {
      setSelected(i,set);
    }
     ++it;
  }
  QListViewItem *i = findItem("..",0);
  if(i)
    setSelected(i,false);
}
QString kcListView::getExpression()
{
  dlgSelect *dlg = new dlgSelect();
  dlg->exec();
  if(!dlg->result())
    return "";
  QString s = dlg->resultString();
  delete dlg;
  return strdup(s);
}
void kcListView::makeViewChange()
{
  emit(changeViewer(name()));
}

void  kcListView::makePropertiesForFileChange()
{
  if(this->currentItem()->text(0) == "..") return;
  QFileInfo inf(directory+this->currentItem()->text(0));
  emit(changePropertiesForFile(inf));
}

void  kcListView::renameFile()
{
  if(!currentItem())
    return;
  if(this->currentItem()->text(0) == "..")
    return;
  renFileInfo = QFileInfo(directory + currentItem()->text(0));
  this->currentItem()->startRename(0);
}
void kcListView::itemWithNewName(QListViewItem * item,int, const QString & name )
{
  QString newName = directory + name;
  QString oldName = renFileInfo.absFilePath();
  debug("Itemtext = " + item->text(0));
  debug("Alter Name = " + oldName);
  debug("Neuer Name = " + newName);
  debug("name = " + name);
  if(QFile(newName).exists())
  {
    QString old = oldName.right(oldName.length() - oldName.findRev("/",-1) -1);
    item->setText(0,old);
    QMessageBox::information( this, "Rename Error", "File\n" + old + "\nexists. Can't rename file\n" + name);
  }
  else
  {
    // ARNO:
    // Ein Filename mit Leer/Sonderzeichen muss konvertiert werden. QUrl?? ich hab keinen Plan...
    debug("mv "+ oldName + " " + newName);
    system("mv "+ oldName + " " + newName);
// Reread mssten wir uns stecken lassen knnen. Neu Sortieren ginge schneller
//  reRead();
    sort();
  }
}

void kcListView::setShowDotFiles()
{
  QSettings settings;
  showDotFiles = !showDotFiles;
  QString n(name());
  if( n == "ListView1")
      settings.writeEntry("/KCommander/General/DontShowDotfilesLeft/", showDotFiles );
  else
      settings.writeEntry("/KCommander/General/DontShowDotfilesRight/", showDotFiles);
  reRead();
}

void kcListView::makeSymlink()
{
  if(!currentItem()) return;
  if(this->currentItem()->text(0) == "..") return;
  QFileInfo inf(directory+this->currentItem()->text(0));
  emit(makeSymlinkToFile(inf));
}

void kcListView::settingsChanged()
{
      QSettings settings;
      itemItalic = settings.readBoolEntry(QString("/KCommander/ListView/italic/"), true);
      itemBold = settings.readBoolEntry(QString("/KCommander/ListView/bold/"), true);
      recolor = settings.readBoolEntry(QString("/KCommander/ListView/recolor/"), true);

  QString n(name());
  if( n == "ListView1")
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView1Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView1Highlight/"), "#ff0000");
  }
  else
  {
    bg = settings.readEntry(QString("/KCommander/Colors/ListView2Background/"), "#c0c0c0");
    hl = settings.readEntry(QString("/KCommander/Colors/ListView2Highlight/"), "#ff0000");
  }
  clipper()->setPaletteBackgroundColor(bg) ;
}

void kcListView::sendFileAsMail()
{
  if(!currentItem()) return;
  if(this->currentItem()->text(0) == "..") return;

    QSettings settings;
    QString ask = settings.readEntry("/KCommander/Mail/Mailusername","none");
    if(ask == "none")
    {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("No valid username given.\nTake a look at the optionsdialog \nand make the right informations please."), 0, 0 );
    	return;
    }
    QString ask1 = settings.readEntry("/KCommander/Mail/Mailserver","none");
    if(ask1 == "none")
    {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("No valid mailserver given.\nTake a look at the optionsdialog \nand make the right informations please."), 0, 0 );
    	return;
    }

  QFileInfo inf(directory+this->currentItem()->text(0));

   if(inf.isDir())
   {
    	QMessageBox::critical ( (QWidget *)this, "Mail", tr("It is not yet possible to send folders.\n\nHint: Make an archive and send this one.\n"), 0, 0 );
    	return;
   }

  mailComposer *c = new mailComposer(inf);
  c->from->setText(ask);    // mit der Mailingliste geht das irgendwie noch nicht :(
  c->from->setEnabled(false);
  c->resize(500,300);
  c->show();
}
