/****************************************************************************
** $Id: kcsmtp.cpp,v 1.13 2002/04/21 17:19:51 cfricke Exp $
**
** Copyright (C) 1992-2000 Trolltech AS.  All rights reserved.
**
** This file is part of an example program for Qt.  This example
** program may be used, distributed and modified without limitation.
**
*****************************************************************************/

#include "kcsmtp.h"

#include <qtextstream.h>
#include <qsocket.h>
#include <qdns.h>
#include <qtimer.h>
#include <qapplication.h>
#include <qmessagebox.h>
#include <qsettings.h>


void replace( QString& str, const QString& before, const QString& after )
{
    if ( before.length() == 0 )
        return;

    int maxsize;
    if ( before.length() > after.length() )
        maxsize = str.length();
    else
        maxsize = ( str.length() / before.length() ) * after.length();

    QChar *buf = new QChar[maxsize + 1]; // + 1 in case maxsize is 0
    const QChar *strBuf = str.unicode();

    int prev = 0;
    int cur = 0;
    int i = 0;
    bool changed = FALSE;

    while ( (uint) (cur = str.find(before, prev)) < str.length() ) {
        if ( cur > prev ) {
            memcpy( buf + i, strBuf + prev, sizeof(QChar) * (cur - prev) );
            i += cur - prev;
        }
        if ( after.length() > 0 ) {
            memcpy( buf + i, after.unicode(), sizeof(QChar) * after.length() );
            i += after.length();
        }
        prev = cur + before.length();
        changed = TRUE;
    }

    if ( changed ) {
        memcpy( buf + i, strBuf + prev,
		sizeof(QChar) * (str.length() - prev) );
        i += str.length() - prev;
        str = QString( buf, i );
    }
    delete[] buf;
}
//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Normales Senden einer TextNachricht
kcSmtp::kcSmtp( const QString &from, const QString &to, const QString &subject, const QString &body )
{
    QSettings settings;
    firstPopLogin = settings.readBoolEntry("/KCommander/Mail/PopLoginFirst",true);

    if(firstPopLogin)
    {
      userName =  settings.readEntry("/KCommander/Mail/Mailusername","none");
      passwd =  settings.readEntry("/KCommander/Mail/MailPasswd","none");
    };

    socket = new QSocket( this );
    socketPop = new QSocket( this );

    connect ( socket, SIGNAL( readyRead() ),
	      this, SLOT( readyRead() ) );
    connect ( socket, SIGNAL( connected() ),
	      this, SLOT( connected() ) );
    connect ( socketPop, SIGNAL( readyRead() ),
	      this, SLOT( readyRead() ) );
    connect ( socketPop, SIGNAL( connected() ),
	      this, SLOT( connected() ) );

	 dnsLookupHelper(); //War mal anders aber wir haben uns entschieden die User infos auszuwerten um Mails zu senden
	                            // weil das mit den freien mailer nur selten geht !! Funktionsname wird spaeter geaendert

//fuer das File wird es spaeter aufgesplittet !! Weil dann muss ja das File eingebettet werden in die Message
    message = QString::fromLatin1( "From: " ) + from +
         QString::fromLatin1( "\nTo: " ) + to +
	      QString::fromLatin1( "\nSubject: " ) + subject +
	      QString::fromLatin1( "\n\n" ) + body + "\n";
    replace( message, QString::fromLatin1( "\n" ), QString::fromLatin1( "\r\n" ) );
    replace( message, QString::fromLatin1( "\r\n.\r\n" ), QString::fromLatin1( "\r\n..\r\n" ) );

    this->from = from;
    rcpt = to;

    state = Init;
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////////
// Senden eines Files !
kcSmtp::kcSmtp( const QString &from, const QString &to, const QString &filePathName )
{
    QFileInfo finfo(filePathName);
    QString fileName = finfo.fileName();

    QSettings settings;
    firstPopLogin = settings.readBoolEntry("/KCommander/Mail/PopLoginFirst",true);

    if(firstPopLogin)
    {
      userName =  settings.readEntry("/KCommander/Mail/Mailusername","none");
      passwd =  settings.readEntry("/KCommander/Mail/MailPasswd","none");
    };

    socket = new QSocket( this );
    socketPop = new QSocket( this );

    connect ( socket, SIGNAL( readyRead() ),
	      this, SLOT( readyReadForFile() ) );
    connect ( socket, SIGNAL( connected() ),
	      this, SLOT( connected() ) );
    connect ( socketPop, SIGNAL( readyRead() ),
	      this, SLOT( readyReadForFile() ) );
    connect ( socketPop, SIGNAL( connected() ),
	      this, SLOT( connected() ) );

	 dnsLookupHelper(); //War mal anders aber wir haben uns entschieden die User infos auszuwerten um Mails zu senden
	                            // weil das mit den freien mailer nur selten geht !! Funktionsname wird spaeter geaendert

	      messagePart1 = QString::fromLatin1("Content-Type: multipart/mixed;\n") +
	      QString::fromLatin1(" boundary=\"------------0A26F574DD17247FC05CCD7C\"\n") + // Achtung Leerzeichen needed !!
	      QString::fromLatin1("MIME-Version: 1.0\n") +
         QString::fromLatin1( "From: " ) + from + // Das ist der Teil vor dem File !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!
	      QString::fromLatin1( "\nTo: " ) + to +
	      QString::fromLatin1( "\nSubject: <" ) + "File: "+ fileName + " sent by KCommander3 >\n\n"+
//	      QString::fromLatin1( "\n\n" ) + fileName + "\n";       // Hier kommt der Body rein, der Noch zusammengestellt werden muss

// Vorbereiten des Attachements !	      	
      	QString::fromLatin1("--------------0A26F574DD17247FC05CCD7C\n") +
      	QString::fromLatin1("Content-Type: text/plain; charset=us-ascii\n") +
      	QString::fromLatin1("Content-Transfer-Encoding: 8bit\n\n") +
      	QString::fromLatin1(".-----------------------------------------( written by R. Mrten & Ch. Fricke )----------\n") +
      	QString::fromLatin1("| This file was sent by a friend using KCommander2.\n") +
      	QString::fromLatin1("| Take a look at http://www.kcommander.org and let us know if all ok with our KCommander.\n") +
      	QString::fromLatin1("----------------------------------------------------------------------------------------\n") +
      	QString::fromLatin1("Here comes the attachement: ") + fileName + QString::fromLatin1(" \n\n") +
      	
      	QString::fromLatin1("--------------0A26F574DD17247FC05CCD7C\n") +  // jeder Teil der Mail ist mit diesem Teil abzuschliessen,
	      // so das man nacher auch mehrere Files senden kann !!!!!
//------------------------------------------------------------------------------------------------
// Jetzt kommt der wichtigste Teil der Message !!   Ein File ist zu senden :)
//------------------------------------------------------------------------------------------------
      	QString::fromLatin1("Content-Type: application/octet-stream;")+QString::fromLatin1(" name=\"") + fileName + QString::fromLatin1("\"\n") +
      	QString::fromLatin1("Content-Transfer-Encoding: base64\n") +
      	QString::fromLatin1("Content-Disposition: attachment;\n") + QString::fromLatin1(" filename=\"") + fileName + QString::fromLatin1("\"\n\n");

    replace( messagePart1, QString::fromLatin1( "\n" ), QString::fromLatin1( "\r\n" ) );
    replace( messagePart1, QString::fromLatin1( "\r\n.\r\n" ), QString::fromLatin1( "\r\n..\r\n" ) );

	
	
	messagePart2 = QString::fromLatin1("\n\n--------------0A26F574DD17247FC05CCD7C--\n\n"); // Das ist der Abschluss der Mail !!!!  Achte auf die -- Zeichen am Ende :)
   replace( messagePart2, QString::fromLatin1( "\n" ),   QString::fromLatin1( "\r\n" ) );
    replace( messagePart1, QString::fromLatin1( "\r\n.\r\n" ), QString::fromLatin1( "\r\n..\r\n" ) );

    this->from = from;
    rcpt = to;
    state = Init;
    prepareFile(filePathName);
}

kcSmtp::~kcSmtp()
{
    delete t;
    delete tPop;
    delete socket;
    delete socketPop;
}

void kcSmtp::connected()
{
   if(! firstPopLogin) emit status( tr( "Connected to %1" ).arg( socket->peerName() ) );
   if(firstPopLogin) emit status( tr( "Connected to %1" ).arg( socketPop->peerName() ) );
}

void kcSmtp::readyRead()
{
    // SMTP is line-oriented
    if ( !socket->canReadLine() )
	return;
    // POP too
    if ( !socketPop->canReadLine() )
	return;

	if(firstPopLogin)  // Wenn wir uns also erstmal auf den Pop-Server connecten sollen !
	{
	  popLogin();
	  return; //Wichtig !! Muss unbedingt stehen bleiben ! Weil nachfolgend kein ELSE steht :)
	}
    QString responseLine;
    do {
	responseLine = socket->readLine();
	response += responseLine;
    } while( socket->canReadLine() && responseLine[3] != ' ' );
    responseLine.truncate( 3 );

    if ( state == Init && responseLine[0] == '2' ) {
	// banner was okay, let's go on
	*t << "HELO there\r\n";
	state = Mail;
    } else if ( state == Mail && responseLine[0] == '2' ) {
	// HELO response was okay (well, it has to be)
	*t << "MAIL FROM: <" << from << ">\r\n";
	state = Rcpt;
    } else if ( state == Rcpt && responseLine[0] == '2' ) {
	*t << "RCPT TO: <" << rcpt << ">\r\n";
	state = Data;
    } else if ( state == Data && responseLine[0] == '2' ) {
    emit status( tr( "Send mail" ) );
	*t << "DATA\r\n";
	state = Body;
    } else if ( state == Body && responseLine[0] == '3' ) {
	*t << message << ".\r\n";
	state = Quit;
    } else if ( state == Quit && responseLine[0] == '2' ) {
	*t << "QUIT\r\n";
	// here, we just close.
	state = Close;
	emit status( tr( "Message sent" ) );
    } else if ( state == Close ) {	// we ignore it
    } else {
	// something broke.
	QMessageBox::warning( qApp->activeWindow(),
			      tr( "Mail sender" ),
			      tr( "Unexpected reply from SMTP server:\n\n" ) +
			      response );
	state = Close;
    }
    response = "";
    if ( state == Close ) {
	QTimer::singleShot( 0, this, SLOT(deleteMe()) );
    }
}

void kcSmtp::deleteMe()
{
    delete this;
}

void kcSmtp::dnsLookupHelper()
{
    QSettings settings;
    QString ask = settings.readEntry("/KCommander/Mail/Mailserver","none");
    QString ask1 = settings.readEntry("/KCommander/Mail/Mailserver_In","none");

      socket->connectToHost( ask, 25 );  // fertig zum Mailsenden
//    if(! firstPopLogin )socket->connectToHost( ask, 25 );  // fertig zum Mailsenden
//    else
//    {
      socketPop->connectToHost(ask1,110);   // entweder hat er schon gePOPped, muss es noch tun oder war nicht konfiguriert !
//      return;
//    }

    t = new QTextStream( socket );
    tPop = new QTextStream( socketPop );
}

void kcSmtp::popLogin()
{
        QString responseLine;
        do
        {
      	  responseLine = socketPop->readLine();
       	  response += responseLine;
        }
        while( socketPop->canReadLine() && responseLine[3] != ' ' );

        responseLine.truncate( 3 );

        if ( state == Init && responseLine == "+OK" ) {
      	   *tPop << "USER " << userName <<"\r\n";
      	   state = Mail;
        } else if ( state == Mail && responseLine == "+OK" ) {
      	   *tPop << "PASS " << passwd << "\r\n";
          	state = Quit;
        } else if ( state == Quit && responseLine == "+OK" ) {
    	      *tPop << "QUIT\r\n";
    	      state = PopReady;
    	      emit status( tr( "Poplogin ready" ) );
        } else if ( state == PopReady ) {
        } else
        {
      	  // something broken
      	  QMessageBox::warning( qApp->activeWindow(),
    			      tr( "Mail sender" ),
    			      tr( "Unexpected reply from Pop server:\n\n" ) +
    			      response );
      	  state = PopReady;
        }
        response = "";
       if ( state == PopReady )
       {
          firstPopLogin = false;  // fertig
          state = Init; // gehe weiter beim SMTP
       }
       return;
}

void kcSmtp::readyReadForFile()
{
    // SMTP is line-oriented
    if ( !socket->canReadLine() )
	return;
    // POP too
    if ( !socketPop->canReadLine() )
	return;

	if(firstPopLogin)  // Wenn wir uns also erstmal auf den Pop-Server connecten sollen !
	{
	  popLogin();
	  return; //Wichtig !! Muss unbedingt stehen bleiben ! Weil nachfolgend kein ELSE steht :)
	}
	
    QString responseLine;
    do {
	responseLine = socket->readLine();
	response += responseLine;
    } while( socket->canReadLine() && responseLine[3] != ' ' );
    responseLine.truncate( 3 );

    if ( state == Init && responseLine[0] == '2' ) {
	// banner was okay, let's go on
	*t << "HELO there\r\n";
	state = Mail;
    } else if ( state == Mail && responseLine[0] == '2' ) {
	// HELO response was okay (well, it has to be)
	*t << "MAIL FROM: <" << from << ">\r\n";
	state = Rcpt;
    } else if ( state == Rcpt && responseLine[0] == '2' ) {
	*t << "RCPT TO: <" << rcpt << ">\r\n";
	state = Data;
    } else if ( state == Data && responseLine[0] == '2' ) {
    emit status( tr( "Send mail" ) );
	*t << "DATA\r\n";
	state = Body;
    } else if ( state == Body && responseLine[0] == '3' ) {
	*t << messagePart1;

    	writeFile(*t);                              //File reinschieben bitte !
	
	*t << messagePart2 << ".\r\n";
	state = Quit;
    } else if ( state == Quit && responseLine[0] == '2' ) {
	*t << "QUIT\r\n";
	// here, we just close.
	state = Close;
	emit status( tr( "Message sent" ) );
    } else if ( state == Close ) {
	// we ignore it
    } else {
	// something broke.
	QMessageBox::warning( qApp->activeWindow(),
			      tr( "Mail sender" ),
			      tr( "Unexpected reply from SMTP server:\n\n" ) +
			      response );
	state = Close;
    }

    response = "";

    if ( state == Close ) {
    socket->close();
	QTimer::singleShot( 0, this, SLOT(deleteMe()) );
    }
}

void kcSmtp::prepareFile(QString filePathName)         //Pfadname bitte nicht vergessen
{
	QFile file("/tmp/.kcmail");
	QProcess prog;
	prog.addArgument("mimencode");
	prog.addArgument(filePathName);
	prog.addArgument("-o");
	prog.addArgument(file.name());
	prog.start();
}

void kcSmtp::writeFile(QTextStream &origStream)
{
  connect(socket,SIGNAL(bytesWritten(int)),this,SLOT(socketHadWritten(int)));

  QFile f("/tmp/.kcmail");
  fsize = f.size();
  if(fsize ==0)
    fsize = 1;  //division 0 error !

  if ( f.open(IO_ReadOnly) )
  {
    QTextStream t( &f );        // use a text stream
    QString s;
    while ( !t.atEnd() )
    { // until end of file...
      s = t.readLine();       // line of text excluding '\n'
      origStream << QString::fromLatin1("\r\n") << s ;// << QString::fromLatin1("\r\n");
    	emit status( tr( "Wait while sending :" )+ QString(" %1 Bytes").arg(fsize) );
    }
    f.close();
    origStream.device()->flush();
  }
  disconnect(socket,SIGNAL(bytesWritten(int)),this,SLOT(socketHadWritten(int)));
}

void kcSmtp::socketHadWritten(int bytes)
{
  socketWritten = true;
}
