#include "kctextviewer.h"

#include <qstringlist.h>
#include <qimage.h>
#include <qfileinfo.h>
#include <qpopupmenu.h>
#include <qmenubar.h>

//*************************************************************************************
kcTextViewer::kcTextViewer(QString filename, int type, QWidget *parent, const char *name ) : QMainWindow(parent,name)
{
  statusbar = new QStatusBar(this);
  stat_fsize = new statusLineEntry(statusBar());
  stat_filepos = new statusLineEntry(statusBar());
  statusbar->addWidget(stat_fsize,200,true);
  statusbar->addWidget(stat_filepos,200,true);
  statusbar->hide();

  hexviewer = new hexViewEdit(filename,false,this);
  connect(hexviewer, SIGNAL(info1changed(QString)), SLOT(setInfo1(QString)));
  connect(hexviewer, SIGNAL(info2changed(QString)), SLOT(setInfo2(QString)));
  connect(hexviewer, SIGNAL(closeParent()), SLOT(close()));

  textviewer = new QTextBrowser(this);
  imageviewer = new ImageViewer(false,this);

	textviewer->close();
	hexviewer->close();
	imageviewer->close();

  setCaption("KCommander::View");
  setName(filename);
  file = filename;

  QPopupMenu* fileMenu = new QPopupMenu( this );
  fileMenu->insertItem( tr("&View as plain text"), this, SLOT( showAsText() ), CTRL+Key_T );
  fileMenu->insertItem( tr("&View as hex"), this, SLOT( showAsHex() ), CTRL+Key_H );
  fileMenu->insertItem( tr("&View the pic"), this, SLOT( showAsPic() ), CTRL+Key_P,99 );
  fileMenu->setItemEnabled(99,false);
  fileMenu->insertSeparator();
  fileMenu->insertItem( tr("&Close"), this, SLOT( close() ), CTRL+Key_Q );
  menuBar()->insertItem( tr("&File"), fileMenu );
	
	QFileInfo info(filename);
	QString tmp(info.extension(true));
	tmp = tmp.lower();
	if(tmp == "jpg" ||tmp == "jpeg" ||tmp == "png" ||tmp == "bmp" ||tmp == "xpm" ||tmp == "gif")
	{
	  imageviewer->loadImage(filename);
     setCentralWidget(imageviewer);
     fileMenu->setItemEnabled(99,true);
     imageviewer->show();
	}
	else	if(tmp == "txt" ||tmp == "htm" ||tmp == "html" ||tmp == "" ||tmp == "xml")
	{
	   if(type == 3) type =0;
		QStringList strList;
		strList.append(".");
    	textviewer->mimeSourceFactory()->setFilePath(strList);
		textviewer->mimeSourceFactory()->setExtensionType("htm", "text/html;charset=iso8859-1");
		textviewer->mimeSourceFactory()->setExtensionType("txt", "text/plain");
		textviewer->mimeSourceFactory()->setExtensionType("xml", "text/xml;charset=UTF-8");
		textviewer->mimeSourceFactory()->setExtensionType("html", "text/html;charset=iso8859-1");
		textviewer->mimeSourceFactory()->setExtensionType("", "text/plain");

  	textviewer->setSource(filename);
   setCentralWidget(textviewer);
   textviewer->show();
  }
	else	if(tmp == "exe" ||tmp == "com" ||tmp == "bin" ||tmp == "wav")
	{
	   if(type == 3) type =0;
	  showAsHex();
  }
//  else if(info.extension(true) == "wav")
//  {
//    if (QSound::isAvailable())
//    QSound::play(filename);
//  }
  else
  {
	  QFile f( file );
	    if ( f.open( IO_ReadOnly ) )
	    {
	        QTextStream ts( &f );
	        textviewer->setText( ts.read());
	    }
      setCentralWidget(textviewer);
      textviewer->show();
	}

  if(type == 1) showAsText();
  if(type == 2) showAsHex();
  if(type == 3) showAsPic();
	
  resize(640,480);
}
kcTextViewer::~kcTextViewer()
{
}
void  kcTextViewer::resizeEvent(QResizeEvent *e)
{
//  textviewer->setGeometry(0,40,width(),height()-40);
//  statusBar()->message("Loading...");
}
void  kcTextViewer::closeEvent(QCloseEvent *e)
{
  e->accept();
}
void  kcTextViewer::mySlot()
{
}
void  kcTextViewer::keyPressEvent(QKeyEvent *e)
{
  if(e->ascii() == 27)
  {
    // close if ESC was pressed
    emit closed();
    e->accept();
    close();
  }
  e->ignore();
}

void  kcTextViewer::showAsText()
{
	setCentralWidget(textviewer);
	textviewer->close();
	imageviewer->close();
	hexviewer->close() ;
   statusbar->hide();
    textviewer->setTextFormat(PlainText);
	  	  			
	  QFile f( file );
	    if ( f.open( IO_ReadOnly ) )
	    {
	        QTextStream ts( &f );
	        textviewer->setText( ts.read(),"hhhhh");
	        textviewer->show();
	    }
}

void  kcTextViewer::showAsHex()
{
	setCentralWidget(hexviewer);
	textviewer->close();
	imageviewer->close();
	
  statusbar->show();
  hexviewer->show();
  hexviewer->setFocus();
}

void  kcTextViewer::showAsPic()
{
	setCentralWidget(imageviewer);
	textviewer->close();
	hexviewer->close();
	imageviewer->close();
   statusbar->hide();

  imageviewer->loadImage(file);
  imageviewer->show();
}

void kcTextViewer::setInfo1(QString s)
{
  stat_fsize->setText(s);
}
void kcTextViewer::setInfo2(QString s)
{
  stat_filepos->setText(s);
}
